@@ -6,24 +6,36 @@ import sys
 sys.path.append('e:\python\libs')
 sys.path.append('c:\python\libs')
 
+import time
+import math
+import md5
+
 import sysinfo
 import e32
 import inbox
-import socket
 import contacts
 import telephone
 import messaging
-import sysinfo
-import md5
-import math
 from appuifw import *
 from status_numbers import *
 
-VERSION = 0.4
+if float(e32.pys60_version[:3]) >= 1.9:
+    import e32calendar as calendar
+else:
+    import calendar
+
+if float(e32.pys60_version[:3]) >= 1.9:
+    import btsocket as socket
+else:
+    import socket
+
+
+VERSION = 0.5
 PORT = 18
 
 # For hash functions
 CONTACT_SEP = chr(0x1F) # Unit Separator
+ENTRY_SEP = chr(0x1F) # Unit Separator
 FIELD_SEP = chr(0x1E) # Record Separator
 INFO_SEP = chr(0x1D) # Group Separator
 
@@ -31,12 +43,13 @@ class Mobile(object):
     def __init__(self):
         self.connected = False
         self.service = False
-        self.useCanvas = False
+        self.useCanvas = True
         self.client = None
 
         self.inbox = inbox.Inbox(inbox.EInbox)
         self.sent = inbox.Inbox(inbox.ESent)
         self.contactDb = contacts.open()
+        self.calendarDb = calendar.open()
         self.currentMessage = None
         self.__partialMessage = ""
 
@@ -48,7 +61,7 @@ class Mobile(object):
         self.initUi()
         self.startService()
 
-    def initUi(self):        
+    def initUi(self):
         app.title = u"Series 60 - Remote"
 
         if self.useCanvas:
@@ -80,7 +93,7 @@ class Mobile(object):
         self.sock = socket.socket(socket.AF_BT, socket.SOCK_STREAM)
         self.sock.bind(('', PORT))
         self.sock.listen(1)
-        
+
         socket.set_security(self.sock,  socket.AUTH | socket.AUTHOR)
         socket.bt_advertise_service(u"pys60_remote", self.sock, True, socket.RFCOMM)
 
@@ -137,12 +150,8 @@ class Mobile(object):
     def wait(self):
         while(True):
             try:
-                print "reading..."
                 data = self.fis.readline()
-                print "read:",  repr(data)
-                print "len:",  len(data)
             except:
-                print "quit..."
                 self.quit()
                 self.startService()
 
@@ -181,7 +190,7 @@ class Mobile(object):
                 contact = self.contactDb.add_contact()
                 contact.commit()
                 self.send(NUM_CONTACTS_ADD_REPLY_ID,  contact.id)
-            
+
             elif (header == NUM_CONTACTS_DELETE):
                 id = int(message)
                 if id in self.contactDb.keys():
@@ -201,6 +210,81 @@ class Mobile(object):
                 value = unicode(message.split(NUM_SEPERATOR)[3])
                 self.modifyContact("remove",  id,  type,  location,  value)
 
+            elif (header == NUM_CALENDAR_REQUEST_HASH_ALL):
+                self.sendCalendarHash()
+
+            #elif (header == NUM_CALENDAR_REQUEST_HASH_SINGLE):
+            #    self.sendCalendarHashSingle()
+
+            elif (header == NUM_CALENDAR_REQUEST_ENTRY):
+                key = int(message.split(NUM_SEPERATOR)[0])
+                entry = self.calendarDb[key]
+                self.sendCalendarEntry(entry)
+
+            elif (header == NUM_CALENDAR_REQUEST_ENTRIES_ALL):
+                self.sendAllCalendarEntries()
+
+            elif (header == NUM_CALENDAR_ENTRY_DELETE):
+                id = int(message)
+                try:
+                    del self.calendarDb[id]
+                except RuntimeError:
+                    # no such entry
+                    pass
+
+            elif (header == NUM_CALENDAR_ENTRY_CHANGE or header == NUM_CALENDAR_ENTRY_ADD):
+                ms = message.split(NUM_SEPERATOR)
+
+                if (header == NUM_CALENDAR_ENTRY_CHANGE):
+                    id = int(ms[0])
+                elif (header == NUM_CALENDAR_ENTRY_ADD):
+                    type = str(ms[0])
+
+                content = unicode(ms[1])
+                location = unicode(ms[2])
+                #start = float(ms[3]) if ms[3] else 0
+                if ms[3]:
+                    start = float(ms[3])
+                else:
+                    start = 0.0
+                #end = float(ms[4]) if ms[4] else None
+                if ms[4]:
+                    end = float(ms[4])
+                else:
+                    end = None
+                replication = str(ms[5])
+                #alarm = float(ms[6]) if ms[6] else None
+                if ms[6]:
+                    alarm = float(ms[6])
+                else:
+                    alarm = None
+                priority = int(ms[7])
+                repeat_type = str(ms[8])
+                repeat_days = str(ms[9])
+                repeat_exceptions = str(ms[10])
+                #repeat_start = float(ms[11]) if ms[11] else 0
+                if ms[11]:
+                    repeat_start = float(ms[11])
+                else:
+                    repeat_start = 0.0
+                #repeat_end = float(ms[12]) if ms[12] else None
+                if ms[12]:
+                    repeat_end = float(ms[12])
+                else:
+                    repeat_end = None
+                #repeat_interval = int(ms[13]) if ms[13] else 1
+                if ms[13]:
+                    repeat_interval = int(ms[13])
+                else:
+                    repeat_interval = 1
+
+                if (header == NUM_CALENDAR_ENTRY_CHANGE):
+                    self.modifyCalendarEntry(id,  content,  location,  start,  end,  replication,  alarm,  priority,  repeat_type,
+                                        repeat_days,  repeat_exceptions,  repeat_start,  repeat_end,  repeat_interval)
+                elif (header == NUM_CALENDAR_ENTRY_ADD):
+                    self.addCalendarEntry(type,  content,  location,  start,  end,  replication,  alarm,  priority,  repeat_type,
+                                        repeat_days,  repeat_exceptions,  repeat_start,  repeat_end,  repeat_interval)
+
             elif (header == NUM_MESSAGE_REQUEST):
                 lastId = int(message.split(NUM_SEPERATOR)[0])
                 self.sendAllMessages(lastId)
@@ -259,7 +343,7 @@ class Mobile(object):
         contactDict = dict()
         for key in keys:
             contact = self.contactDb[key]
-            
+
             # Check for empty title (please look in the comment for sendContact)
             try:
                 contact.title
@@ -273,7 +357,7 @@ class Mobile(object):
                 value = unicode(value)
                 value = value.replace(u'\u2029',  u'\n') # PARAGRAPH SEPARATOR (\u2029) replaced by LINE FEED (\u000a)
                 location = field.location
-                
+
                 if _type == "unknown":
                     continue
                 elif _type == "thumbnail_image":
@@ -296,20 +380,20 @@ class Mobile(object):
         contacts = self.contactDict()
         keys = contacts.keys()
         keys.sort()
-        
+
         hash = unicode()
-        
+
         for key in keys:
             hash += str(key)
-            hash += FIELD_SEP 
+            hash += FIELD_SEP
             for _type,  location, value in contacts[key]:
                 hash += _type + INFO_SEP + location + INFO_SEP + value
                 hash += FIELD_SEP
             hash += CONTACT_SEP
-        
+
         hash = hash.encode("utf8")
         hash = md5.md5(hash).hexdigest()
-        self.send(NUM_CONTACTS_REPLY_HASH_ALL, hash)        
+        self.send(NUM_CONTACTS_REPLY_HASH_ALL, hash)
 
     def sendContactHashSingle(self):
         self.send(NUM_CONTACTS_REPLY_HASH_SINGLE_START)
@@ -317,17 +401,17 @@ class Mobile(object):
         contacts = self.contactDict()
         keys = contacts.keys()
         keys.sort()
-        
+
         for key in keys:
             hash = unicode()
             for _type,  location, value in contacts[key]:
                 hash += _type + INFO_SEP + location + INFO_SEP + value
                 hash += FIELD_SEP
-            
+
             hash = hash.encode("utf8")
             hash = md5.md5(hash).hexdigest()
             self.send(NUM_CONTACTS_REPLY_HASH_SINGLE_LINE, key,  hash)
-        
+
         self.send(NUM_CONTACTS_REPLY_HASH_SINGLE_END)
 
     def sendAllContacts(self):
@@ -378,7 +462,7 @@ class Mobile(object):
             contact = self.contactDb[id]
         except:
             return
-        
+
         if type == u"thumbnail_image":
             if modification == "remove":
                 self.setContactThumbnail(contact)
@@ -391,9 +475,9 @@ class Mobile(object):
             else:
                 self.setContactBirthday(contact,  value)
             return
-        
+
         contact.begin()
-        
+
         if modification == "add":
             contact.add_field(type,  value,  location=location)
         elif modification == "remove":
@@ -402,12 +486,12 @@ class Mobile(object):
                 if field.value == value:
                     index = field.index
                     break
-            
+
             if index != -1:
                 del contact[index]
-        
+
         contact.commit()
-    
+
     def getDetailFromVcard(self,  contact,  detail,  delimiter='\r\n'):
         # This is an ugly hack, needed for some fields that cannot be handled using the contact object
         try:
@@ -416,16 +500,16 @@ class Mobile(object):
             return value
         except:
             return
-    
+
     def setDetailFromVcard(self,  contact,  detail,  value,  delimiter='\r\n'):
         # This is an ugly hack, needed for some fields that cannot be handled using the contact object
         card = contact.as_vcard()
-        
+
         new = u""
         for line in card.split("\r\n"):
             if line.startswith("BEGIN:") or line.startswith("VERSION:") or line.startswith("REV:") or line.startswith("UID:"):
                 new += line + "\r\n"
-        
+
         # Format value: New line (\r\n) after 64 chars, followed by 4 spaces
         if len(value) > 64:
             fmtvalue = "\r\n"
@@ -436,10 +520,10 @@ class Mobile(object):
 
         new += detail + ":" + fmtvalue + delimiter
         new += "END:VCARD"
-        
+
         changed_contact = self.contactDb.import_vcards(new)[0]
         assert changed_contact.id == contact.id
-    
+
     def getContactThumbnail(self,  contact):
         # Ugly workaround!
         # HACK: The value of type "thumbnail_image" is empty, it is only shown when we export the contact to a vCard
@@ -449,19 +533,298 @@ class Mobile(object):
             image = image.replace("\r",  "").replace("\n",  "").replace(" ",  "")
             return image
         return
-    
+
     def setContactThumbnail(self,  contact,  image=""):
         # Ugly workaround!
-        # HACK: There seems to be new other way to update/add the contact picture        
+        # HACK: There seems to be new other way to update/add the contact picture
         self.setDetailFromVcard(contact,  "PHOTO;TYPE=JPEG;ENCODING=BASE64",  image,  "\r\n\r\n")
 
     def getContactBirthday(self,  contact):
         return self.getDetailFromVcard(contact,  "BDAY")
-    
+
     def setContactBirthday(self,  contact,  date=""):
         # HACK: It isn't possible to set birthdays < year 1970 (before the beginning of the unix epoch)
         self.setDetailFromVcard(contact,  "BDAY",  date)
 
+    def __calendarGetType(self,  entry):
+        if isinstance(entry,  calendar.CalendarDb.AnniversaryEntry):
+            return "anniversary"
+        elif isinstance(entry,  calendar.CalendarDb.AppointmentEntry):
+            return "appointment"
+        elif isinstance(entry,  calendar.CalendarDb.EventEntry):
+            return "event"
+        elif isinstance(entry,  calendar.CalendarDb.ReminderEntry):
+            return "reminder"
+        elif isinstance(entry,  calendar.CalendarDb.TodoEntry):
+            return "todo"
+        return ""
+
+    def __calendarFormatEntry(self,  entry,  sep):
+        entryType = self.__calendarGetType(entry)
+        if not entryType:
+            # Type could also be TodoListDict or TodoList, which is currently not handled...
+            return str()
+
+        line = entryType + sep + entry.content + sep + entry.location + sep
+        if entry.start_time:
+            # None if the start datetime of the entry is not set
+            line += str(long(entry.start_time)) # entry.start_time is of type float
+        line += sep
+
+        if entry.end_time:
+            line += str(long(entry.end_time))
+        line += sep
+
+        line += str(long(entry.last_modified)) + sep
+        line += entry.replication + sep  # open, private or restricted
+
+        if entry.alarm:
+            line += str(long(entry.alarm)) # The alarm datetime value (float) for the entry
+        line += sep
+
+        line += str(entry.priority) + sep
+
+        # Format repeat...
+        repeat = entry.get_repeat()
+        repeated = bool(repeat)
+
+        if repeated:
+            line += repeat["type"] # daily, weekly, monthly_by_dates, monthly_by_days, yearly_by_date, yearly_by_day
+        line += sep
+
+        # It is really hard to extract the days...
+        # We try to seperate the values by ',' and different groups by ';'
+        if repeated and "days" in repeat:
+            days = repeat["days"]
+
+            if isinstance(days,  (list,  tuple)):
+                if len(days) == 0:
+                    pass
+                elif isinstance(days[0],  int):
+                    # Which days in week? (0=Monday,...)
+                    # for example: [0,1,2]
+                    line += ",".join([str(day) for day in days])
+                elif isinstance(days[0],  dict):
+                    # example: on second Tuesday and last Monday of the month
+                    # days is: [{'week': 1, 'day': 1}, {'week': 4, 'day': 0}]
+                    # results in: 'week:1,day:1;week:4,day:0'
+                    line += ";".join([",".join([key + ":" + str(value) for key, value in entry.items()]) for entry in days])
+            elif isinstance(days,  dict):
+                # for example: {'week': 1, 'day': 1, 'month': 1}
+                # results in: 'week:1,day:1,month:1'
+                line += ",".join([key + ":" + str(value) for key, value in days.iteritems()])
+        line += sep
+
+        if repeated and "exceptions" in repeat:
+            line += ",".join([str(int(day)) for day in repeat["exceptions"]])
+        line += sep
+
+        if repeated and "start" in repeat:
+            line += str(long(repeat["start"]))
+        line += sep
+
+        if repeated and "end" in repeat:
+            end = long(repeat["end"])
+            end -= time.timezone
+            if end == 4102441200L:
+                # 4102441200L = 2100-01-01T00:00:00
+                # There is a bug in PyS60, which causes that None as end date throws as error
+                # As workarond, we use 2100-01-01 as end date for eternal repeats
+                # Bug report: http://developer.symbian.org/bugs/show_bug.cgi?id=3666
+                end = ""
+            line += str(end)
+        line += sep
+
+        if repeated and "interval" in repeat:
+            line += str(repeat["interval"])
+        line += sep
+
+        if entryType == "todo":
+            # cross out property is valid only for todo entries
+            # a value that is interpreted as false means that the entry is not crossed out
+            line += str(int(bool(entry.crossed_out))) + sep
+
+            if entry.cross_out_time:
+                line += str(long(entry.cross_out_time))
+            line += sep
+
+            # TODO lists no more supported!
+            #line += entry.todo_list + sep  # The ID of the TODO list to which this entry belongs
+
+        return line
+
+    def calendarDict(self):
+        calendarDict = dict()
+        for key in self.calendarDb:
+            entry = self.calendarDb[key]
+            line = self.__calendarFormatEntry(entry,  FIELD_SEP)
+            if not line:
+                continue
+
+            calendarDict[entry.id] =  line
+
+        return calendarDict
+
+    def sendCalendarHash(self):
+        calendarDict = self.calendarDict()
+
+        keys = calendarDict.keys()
+        keys.sort()
+
+        hash = unicode()
+        for key in keys:
+            hash += str(key)
+            hash += FIELD_SEP
+            hash += calendarDict[key]
+            hash += FIELD_SEP
+            hash += ENTRY_SEP
+
+        #self.send(NUM_DEBUG,  "Mobile:::" + hash.replace(FIELD_SEP,  ";").replace(ENTRY_SEP,  "\n"))
+
+        hash = hash.encode("utf8")
+        hash = md5.md5(hash).hexdigest()
+        self.send(NUM_CALENDAR_REPLY_HASH_ALL, hash)
+
+# We don't need to send a hash for every entry, it's better if we send directly the data
+#    def sendCalendarHashSingle(self):
+#        self.send(NUM_CALENDAR_REPLY_HASH_SINGLE_START)
+#
+#        calendarDict = self.calendarDict()
+#        keys = calendarDict.keys()
+#        keys.sort()
+#
+#        print "keys are of length",  len(keys)
+#
+#        for key in keys:
+#            hash = calendarDict[key]
+#            hash = hash.encode("utf8")
+#            hash = md5.md5(hash).hexdigest()
+#
+#            self.send(NUM_CALENDAR_REPLY_HASH_SINGLE_LINE, key,  hash)
+#
+#        self.send(NUM_CALENDAR_REPLY_HASH_SINGLE_END)
+
+    def sendAllCalendarEntries(self):
+        self.send(NUM_CALENDAR_REPLY_ENTRIES_START)
+        for key in self.calendarDb:
+            entry = self.calendarDb[key]
+            self.sendCalendarEntry(entry )
+        self.send(NUM_CALENDAR_REPLY_ENTRIES_END)
+
+    def sendCalendarEntry(self,  entry):
+        self.send(NUM_CALENDAR_REPLY_ENTRY,  entry.id,  self.__calendarFormatEntry(entry,  NUM_SEPERATOR))
+
+    def modifyCalendarEntry(self,  id,  content,  location,  start,  end,  replication,  alarm,  priority, repeat_type,
+                            repeat_days,  repeat_exceptions,  repeat_start,  repeat_end,  repeat_interval):
+
+        try:
+            entry = self.calendarDb[id]
+        except:
+            return
+
+        entry.begin()
+
+        self.modifyCalendarEntryFields(entry,  content,  location,  start,  end,  replication,  alarm,  priority, repeat_type,
+                            repeat_days,  repeat_exceptions,  repeat_start,  repeat_end,  repeat_interval)
+
+        entry.commit()
+
+        self.send(NUM_CALENDAR_ENTRY_CHANGE_REPLY_TIME,  id,  str(long(entry.last_modified)))
+
+    def addCalendarEntry(self,  type,  content,  location,  start,  end,  replication,  alarm,  priority, repeat_type,
+                            repeat_days,  repeat_exceptions,  repeat_start,  repeat_end,  repeat_interval):
+
+        if type == "appointment":
+            entry = self.calendarDb.add_appointment()
+        elif type == "event":
+            entry = self.calendarDb.add_event()
+        elif type == "anniversary":
+            entry = self.calendarDb.add_anniversary()
+        elif type == "todo":
+            entry = self.calendarDb.add_todo()
+        else:
+            return
+
+        self.modifyCalendarEntryFields(entry,  content,  location,  start,  end,  replication,  alarm,  priority, repeat_type,
+                            repeat_days,  repeat_exceptions,  repeat_start,  repeat_end,  repeat_interval)
+
+        entry.commit()
+
+        self.send(NUM_CALENDAR_ENTRY_ADD_REPLY,  entry.id,  str(long(entry.last_modified)))
+
+    def modifyCalendarEntryFields(self,  entry,  content,  location,  start,  end,  replication,  alarm,  priority, repeat_type,
+                            repeat_days,  repeat_exceptions,  repeat_start,  repeat_end,  repeat_interval):
+        entry.content = content
+        entry.location = location
+        entry.set_time(start,  end)
+        entry.replication = replication
+        entry.alarm = alarm
+        entry.priority = priority
+        
+        # Clear the old repeat
+        entry.set_repeat(None)
+        
+        entry.set_repeat(self.buildCalendarEntryRepeat(repeat_type,  repeat_days,  repeat_exceptions,  repeat_start,  repeat_end,  repeat_interval))
+    
+    def buildCalendarEntryRepeat(self,   type,  days,  exceptions,  start,  end,  interval):
+        if not type:
+            return None
+        if type not in ("weekly",  "monthly_by_dates",  "monthly_by_days",  "yearly_by_day",  "yearly_by_date"):
+            return None
+
+        days = self.calendarEntryParsedDays(type,  days)
+        if exceptions:
+            exceptions = [float(exception) for exception in exceptions.split(',')]
+        else:
+            exceptions = []
+
+        if end is None:
+            # 4102441200. = 2100-01-01T00:00:00
+            # There is a bug in PyS60, which causes that None as end date throws as error
+            # As workarond, we use 2100-01-01 as end date for eternal repeats
+            # Bug report: http://developer.symbian.org/bugs/show_bug.cgi?id=3666
+            # We also can't use time.mktime([2100, 1, 1, 0, 0, 0, 1, 1, -1]), cause this wouls result in
+            # an integer overflow
+            end = 4102441200.
+
+        # We need to subtract one day from the end date, because
+        # endTime+=TTimeIntervalDays(1);
+        # in calendarmodule.h adds one day after set_repeat is called
+        end -= 24*60*60
+
+        return {"type" : type,
+                      "days" : days,
+                      "exceptions" : exceptions,
+                      "start" : start,
+                      "end" : end,
+                      "interval" : interval}
+
+    def calendarEntryParsedDays(self,  type,  days):
+
+        ### PLEASE COPY CHANGES IN THIS FUNCTION ALSO TO
+        ### pc/lib/classes.py, class CalendarEntry function recurrenceParsedDays
+
+        if type in ("weekly",   "monthly_by_dates"):
+            # 0,1,2 -> [0, 1, 2]
+            return [int(day) for day in days.split(',')]
+        if type == "monthly_by_days":
+            # week:1,day:1;week:4,day:0 -> [{'week': 1, 'day': 1}, {'week': 4, 'day': 0}]
+            dates = []
+            for date in days.split(";"):
+                tmp = dict()
+                for sub in date.split(","):
+                    key, value = sub.split(":")
+                    tmp[key] = int(value)
+                dates.append(tmp)
+            return dates
+        if type == "yearly_by_day":
+            # week:1,day:1,month:1 -> {'week': 1, 'day': 1, 'month': 1}
+            dates = {}
+            for subentry in days.split(","):
+                key,  value = subentry.split(":")
+                dates[key] = int(value)
+            return dates
+
     def sendAllMessages(self,  lastId):
         messages = list()
         inbox = list()
@@ -563,7 +926,7 @@ class Mobile(object):
         telephone.EStatusHold: "hold",
         telephone.EStatusTransferring: "transferring",
         telephone.EStatusTransferAlerting: "transfer alerting" }
-        
+
         self.send(NUM_INCOMING_CALL, number,  call_state[state])
 
     def setRead(self,  id,  state):
@@ -575,7 +938,6 @@ class Mobile(object):
         self.inbox.set_unread(id,  state)
 
     def quit(self):
-        print "in quit function"
         if (self.service):
             self.service = False
 
@@ -605,24 +967,24 @@ class Mobile(object):
 
 # Debug of SIS applications
 try:
-   mobile = Mobile()
+    mobile = Mobile()
 except Exception, e:
     # Oops, something wrong. Report problems to user
     # and ask him/her to send them to you.
     import traceback
 
     new_line = u"\u2029"
- 
+
     # Collecting call stack info
     info = sys.exc_info()
-    
+
     # Show the last 4 lines of the call stack
     call_stack = u""
-    for file, lineno, function, text in traceback.extract_tb(info[2])[:4]:
+    for file, lineno, function, text in traceback.extract_tb(info[2])[4:]:
         call_stack += file + u": " + str(lineno) + u" - " + function + new_line
         call_stack += u" " + repr(text) + new_line
     call_stack +=  u"%s: %s" % info[:2]
- 
+
     # Creating a friendly user message with exception details
     err_msg = u"This programs was unexpectedly closed due to the following error: "
     err_msg += unicode(repr(e)) + new_line
@@ -630,7 +992,7 @@ except Exception, e:
     err_msg += u"send it to series60-remote-devel@lists.sourceforge.net. "
     err_msg += u"Thanks in advance and sorry for this inconvenience." + new_line*2
     err_msg += u"Call stack:" + new_line + call_stack
- 
+
     # Small PyS60 application
     lock = e32.Ao_lock()
     app.body = Text(err_msg)
