@@ -60,15 +60,19 @@ class Connection(QObject):
         self.__saveAllMessages = saveAllMessages
 
         self.__connected = False
-        self.__scan = ScanDevices(self)
+        self.__scan = ScanDevices(self,  True)
         self.__listenThread = None
         self.__connectionStates = 4
-    
+
     def __str__(self):
         return "\"Connection with series60\""
 
+    def scanner(self):
+        """Return an instance of a device scanner"""
+        return ScanDevices(self)
+
     def scan(self):
-        """Scan for devices"""        
+        """Scan for devices"""
         self.__scan.start()
 
     def scanStop(self):
@@ -94,6 +98,7 @@ class Connection(QObject):
         self.__connectionEstablishment = True
         self.__device = device
         self.__contacts = dict()
+        self.__calendarEntries = dict()
         self.__contactHashes = dict()
         self.__addedContacts = list()
         self.__changedContacts = list()
@@ -108,6 +113,7 @@ class Connection(QObject):
         self.__messagesSent = list()
         self.__messagesPending = list()
         self.__contactAddQueue = list()
+        self.__calendarAddQueue = list()
 
         if USE_PYBLUEZ:
             self.connect(self.__listenThread,  SIGNAL("finished()"),  self._lostConnection)
@@ -138,6 +144,7 @@ class Connection(QObject):
 
         elif header == NUM_HELLO_REPLY:
             self.emit(SIGNAL("connectionStateChanged"),  2)
+            self.emit(SIGNAL("connectionEstablished"))
 
             if self.__connectionEstablishment:
                 self.refreshSysinfo()
@@ -186,7 +193,10 @@ class Connection(QObject):
             else:
                 self.__log.info("Checksum of contacts on device and pc are equal")
                 self.emit(SIGNAL("contactsCompleted"))
-                self.__connectionCompleted()
+                self.emit(SIGNAL("connectionStateChanged"),  4)
+
+                # The contacts are updated, let's get the calendar entries
+                self.__requestCalendar()
 
         elif header == NUM_CONTACTS_REPLY_HASH_SINGLE_START:
             self.__contactHashes = dict()
@@ -197,7 +207,7 @@ class Connection(QObject):
             self.__contactHashes[id] = hash
 
         elif header == NUM_CONTACTS_REPLY_HASH_SINGLE_END:
-            self.__checkHashes()
+            self.__checkContactHashes()
             self.__contacts = dict()
 
         elif header == NUM_CONTACTS_REPLY_CONTACT_START:
@@ -226,11 +236,75 @@ class Connection(QObject):
 
             if not (self.__addedContacts  or self.__changedContacts or self.__waitForContactEndReply):
                 self.emit(SIGNAL("contactsCompleted"))
-                self.__connectionCompleted()
+                self.emit(SIGNAL("connectionStateChanged"),  4)
+                self.__requestCalendar()
 
         elif header == NUM_CONTACTS_REPLY_CONTACTS_ALL_END:
             self.emit(SIGNAL("contactsCompleted"))
-            self.__connectionCompleted()
+            self.emit(SIGNAL("connectionStateChanged"),  4)
+            self.__requestCalendar()
+
+        elif header == NUM_CALENDAR_REPLY_HASH_ALL:
+            mobileHash = message
+            pcHash = self.__database.calendarHash()
+
+            self.__log.info(QString("Calendar hash from device is %1 and pc hash is %2").arg(mobileHash).arg(pcHash))
+
+            if mobileHash != pcHash:
+                self.__log.info("Checksum of calendar entries aren't equal, requesting all entries...")
+                self.__send(NUM_CALENDAR_REQUEST_ENTRIES_ALL)
+            else:
+                self.__log.info("Checksum of calendar entries on device and pc are equal")
+                self.emit(SIGNAL("calendarCompleted"))
+
+                # We updated the contact and calendar database, so the connection is completed ;)
+                self.__connectionCompleted()
+
+        elif header == NUM_CALENDAR_REPLY_ENTRIES_START:
+            self.__calendarEntries = dict()
+
+        elif header == NUM_CALENDAR_REPLY_ENTRY:
+            entry = CalendarEntry()
+            entry.setDevice(self.device())
+
+            ms = message.split(NUM_SEPERATOR)
+            entry.setIdOnPhone(int(ms[0]))
+            entry.setType(str(ms[1]))
+            entry.setContent(ms[2])
+            entry.setLocation(ms[3])
+            if ms[4]:
+                # Time could be an empty string, would result in:
+                # ValueError: empty string for float()
+                entry.setStartTime(float(ms[4]))
+            if ms[5]:
+                entry.setEndTime(float(ms[5]))
+            if ms[6]:
+                entry.setLastModified(float(ms[6]))
+            entry.setReplication(str(ms[7]))
+            if ms[8]:
+                entry.setAlarm(float(ms[8]))
+            entry.setPriority(int(ms[9]))
+            entry.setRepeatType(ms[10])
+            entry.setRepeatDays(ms[11])
+            if ms[12]:
+                entry.setRepeatExceptions(ms[12])
+            if ms[13]:
+                entry.setRepeatStart(float(ms[13]))
+            if ms[14]:
+                entry.setRepeatEnd(float(ms[14]))
+            if ms[15]:
+                # Repeat interval could be an empty string, this prevents:
+                # ValueError: invalid literal for int() with base 10: ''
+                entry.setRepeatInterval(int(ms[15]))
+            if entry.type() == "todo":
+                entry.setCrossedOut(bool(int(ms[16])))
+                if ms[17]:
+                    entry.setCrossOutTime(float(ms[17]))
+
+            self.__calendarEntries[entry.idOnPhone()]= entry
+
+        elif header == NUM_CALENDAR_REPLY_ENTRIES_END:
+            self.__checkCalendar()
 
         elif header == NUM_MESSAGE_REPLY_LINE:
             msg = Message()
@@ -312,7 +386,29 @@ class Connection(QObject):
             contact.setId(self.__database.contactAdd(contact))
             self.contactChange(contact,  list(),  contact.values())
 
-            self.emit(SIGNAL("contactsCompleted"))
+            self.emit(SIGNAL("contactsUpdated"))
+
+        elif header == NUM_CALENDAR_ENTRY_ADD_REPLY:
+            id = int(message.split(NUM_SEPERATOR)[0])
+            time = int(message.split(NUM_SEPERATOR)[1])
+            time = QDateTime.fromTime_t(time)
+
+            entry = self.__calendarAddQueue.pop(0)
+            entry.setIdOnPhone(id)
+            entry.setLastModified(time)
+            entry.setId(self.__database.calendarEntryAdd(entry))
+
+            self.emit(SIGNAL("calendarUpdated"))
+
+        elif header == NUM_CALENDAR_ENTRY_CHANGE_REPLY_TIME:
+            # This updates the last modified time of an entry in the database (usually after an update of the entry on the PC)
+            id = int(message.split(NUM_SEPERATOR)[0])
+            time = int(message.split(NUM_SEPERATOR)[1])
+            time = QDateTime.fromTime_t(time)
+
+            self.__database.calendarEntryUpdateLastModifiedTime(id,  time)
+
+            self.emit(SIGNAL("calendarUpdated"))
 
         elif header == NUM_DEBUG:
             self.__log.error(QString("Debug data: %1").arg(message))
@@ -320,28 +416,41 @@ class Connection(QObject):
         else:
             self.__log.error(QString("Unknown header: %1 - Data: %2").arg(header).arg(message))
 
+    def __requestCalendar(self):
+        self.__log.debug("The contacts are updated, let's get the calendar entries...")
+        if self.__database.calendarEntriesCount() > 0:
+            self.__send(NUM_CALENDAR_REQUEST_HASH_ALL)
+        else:
+            self.__log.info("No entries saved, requesting all...")
+            self.__send(NUM_CALENDAR_REQUEST_ENTRIES_ALL)
+
     def __connectionCompleted(self):
         if self.__device and self.__device.name():
             self.__connected = True
-            self.emit(SIGNAL("connectionStateChanged"),  4)
+            self.emit(SIGNAL("connectionStateChanged"),  5)
             self.__connectionEstablishment = False
             self.emit(SIGNAL("connectionCompleted"))
             self.__newMessages()
 
-    def __checkHashes(self):
-        pcHashes = self.__database.contactHashSingle()
-        self.__addedContacts = list()
-        self.__changedContacts = list()
-        self.__deletedContacts = list()
-        for id,  hash in self.__contactHashes.iteritems():
+    def __checkHashes(self,  pcHashes,  mobileHashes):
+        addedEntries = list()
+        changedEntries = list()
+        deletedEntries = list()
+        for id,  hash in mobileHashes.iteritems():
             if not id in pcHashes:
-                self.__addedContacts.append(id)
+                addedEntries.append(id)
             elif not hash == pcHashes[id]:
-                self.__changedContacts.append(id)
+                changedEntries.append(id)
 
         for id,  hash in pcHashes.iteritems():
-            if not id in self.__contactHashes:
-                self.__deletedContacts.append(id)
+            if not id in mobileHashes:
+                deletedEntries.append(id)
+
+        return addedEntries,  changedEntries,  deletedEntries
+
+    def __checkContactHashes(self):
+        pcHashes = self.__database.contactHashSingle()
+        self.__addedContacts,  self.__changedContacts,  self.__deletedContacts = self.__checkHashes(pcHashes,  self.__contactHashes)
 
         self.__log.info(QString("Requesting added contacts with id %1").arg(str(self.__addedContacts)))
         self.__requestContacts(self.__addedContacts)
@@ -354,7 +463,53 @@ class Connection(QObject):
 
         if (self.__deletedContacts and not (self.__addedContacts or self.__changedContacts)):
             self.emit(SIGNAL("contactsCompleted"))
-            self.__connectionCompleted()
+            self.emit(SIGNAL("connectionStateChanged"),  4)
+            self.__requestCalendar()
+
+#    def __checkCalendarHashes(self):
+#        pcHashes = self.__database.calendarHashSingle()
+#        self.__addedCalendarEntries,  self.__changedCalendarEntries,  self.__deletedCalendarEntries = self.__checkHashes(pcHashes,  self.__calendarHashes)
+#
+#        self.__log.info(QString("Requesting added calendar entries with id %1").arg(str(self.__addedCalendarEntries)))
+#        self.__requestCalendarEntries(self.__addedCalendarEntries)
+#
+#        self.__log.info(QString("Requesting changed calendar entries with id %1").arg(str(self.__changedCalendarEntries)))
+#        self.__requestCalendarEntries(self.__changedCalendarEntries)
+#
+#        self.__log.info(QString("Removing deleted calendar entries with id %1").arg(str(self.__deletedCalendarEntries)))
+#        self.__database.calendarEntriesRemove(self.__deletedCalendarEntries)
+#
+#        if (self.__deletedCalendarEntries and not (self.__addedCalendarEntries or self.__changedCalendarEntries)):
+#            self.emit(SIGNAL("calendarCompleted"))
+#            self.connectionCompleted()
+
+    def __checkCalendar(self):
+        pcIds = self.__database.calendarIds()
+        mobileIds = self.__calendarEntries.keys()
+
+        for id in mobileIds:
+            if id in pcIds:
+                self.__database.calendarEntryUpdate(self.__calendarEntries[id])
+            else:
+                self.__database.calendarEntryAdd(self.__calendarEntries[id])
+
+        for id in pcIds:
+            if id not in mobileIds:
+                self.__database.calendarEntryRemoveByIdOnPhone(id)
+
+        self.emit(SIGNAL("calendarCompleted"))
+        self.__connectionCompleted()
+
+        # Cleanup
+        self.__calendarEntries = dict()
+
+    def __requestContacts(self,  list):
+        for id in list:
+            self.__send(NUM_CONTACTS_REQUEST_CONTACT,  id)
+
+    def __requestCalendarEntries(self,  list):
+        for id in list:
+            self.__send(NUM_CALENDAR_REQUEST_ENTRY,  id)
 
     def __send(self,  header,  *message):
         new_message = ""
@@ -378,8 +533,8 @@ class Connection(QObject):
                 sentParts += 1
             return
 
-        self.__log.debug(QString("Sent data: Header: %1 - Message: %2").arg(header).arg(new_message))
-        
+        self.__log.debug(QString("Sent data: Header: %1 - Message: %2").arg(header).arg(new_message.replace(NUM_SEPERATOR,  " - ")))
+
         self.emit(SIGNAL("requestSend(QString)"),  QString(str(header) + str(NUM_END_HEADER) + unicode(new_message).encode("utf8") + '\n'))
 
     def __sendMessage(self,  message):
@@ -425,12 +580,12 @@ class Connection(QObject):
                 self.emit(SIGNAL("connectionClosed"),  self.__device.name())
             else:
                 self.emit(SIGNAL("connectionAborted"),  self.__device.name())
-        
+
         self.__connected = False
         self.__newMessagesCompleted = False
         self.__contacts = dict()
         self.__device = Device()
-    
+
     def __newMessages(self):
         if self.__saveAllMessages:
             self.__newMessagesNumber = 0
@@ -438,10 +593,6 @@ class Connection(QObject):
             self.emit(SIGNAL("messagesRequest"))
             self.__send(NUM_MESSAGE_REQUEST,  last)
 
-    def __requestContacts(self,  list):
-        for id in list:
-            self.__send(NUM_CONTACTS_REQUEST_CONTACT,  id)
-
     def sendMessage(self,  message):
         """Send a text message to the contact message.contact()"""
         message.setId(self.__database.messageAdd(message))
@@ -480,6 +631,38 @@ class Connection(QObject):
             value = value.replace(u'\n', u'\u2029') # LINE FEED (\u000a) replaced by PARAGRAPH SEPARATOR (\u2029)
             self.__send(NUM_CONTACTS_CHANGE_ADDFIELD,  contact.idOnPhone(),  field.type(),  field.location(),  value)
 
+    def __calendarEntryAddChangeOptions(self,  entry):
+        content = entry.content()
+        location = entry.location()
+        start = entry.realStartTime().toTime_t() if entry.realStartTime().isValid() else str()
+        end = entry.realEndTime().toTime_t() if entry.realEndTime().isValid() else str()
+        replication = entry.replication()
+        alarm = entry.alarm().toTime_t() if entry.alarm().isValid() else str()
+        priority = entry.priority()
+        repeat_type = entry.repeatType()
+        repeat_days = entry.repeatDays()
+        repeat_exceptions = entry.repeatExceptions()
+        repeat_start = entry.repeatStart().toTime_t() if entry.repeatStart().isValid() else str()
+        repeat_end = entry.repeatEnd().toTime_t() if entry.repeatEnd().isValid() else str()
+        repeat_interval = entry.repeatInterval()
+        return content,  location,  start,  end,  replication,  alarm,  priority,  repeat_type, repeat_days, \
+                    repeat_exceptions,  repeat_start,  repeat_end,  repeat_interval
+
+    def calendarEntryAdd(self,  entry):
+        self.__calendarAddQueue.append(entry)
+        self.__send(NUM_CALENDAR_ENTRY_ADD,  entry.type(),  *self.__calendarEntryAddChangeOptions(entry))
+
+    def calendarEntryChange(self,  entry):
+        id = entry.idOnPhone()
+
+        self.__send(NUM_CALENDAR_ENTRY_CHANGE,  id,  *self.__calendarEntryAddChangeOptions(entry))
+
+        self.__database.calendarEntryUpdate(entry)
+
+    def calendarEntryRemove(self,  entry):
+        self.__send(NUM_CALENDAR_ENTRY_DELETE,  entry.idOnPhone())
+        self.__database.calendarEntryRemove(entry)
+
     def refreshSysinfo(self):
         """Refresh system informations"""
         self.__send(NUM_SYSINFO_REQUEST,  int(not self.__database.deviceCheckValues(self.__device)))
@@ -508,74 +691,130 @@ class Connection(QObject):
 
         if sendQuit:
             self.__send(NUM_QUIT)
-        
+
         self._lostConnection()
 
 if USE_PYBLUEZ:
     if sys.platform == "win32":
-        # Asynchronous mode isn't implemented
-        class ScanDevices(QThread):
-            def __init__(self,  parent):
-                super(ScanDevices,  self).__init__(parent)
-                self.__parent = parent
+        if bluetooth.have_widcomm:
+            # When we use the wicomm bluetooth stack use the asynchronous device scan
+            # This is our only possibility to get the device class
+            class ScanDevices(QThread):
+                def __init__(self,  parent,  emitFromParent=False):
+                    super(ScanDevices,  self).__init__(parent)
+                    self.__parent = parent
+                    if emitFromParent:
+                        self.__emitter = self.__parent
+                    else:
+                        self.__emitter = self
 
-            def stop(self):
-                # TODO: Kill this thread?
-                pass
+                def stop(self):
+                    # TODO: Kill this thread?
+                    self.__emitter.emit(SIGNAL("scanCompleted"))
+
+                def run(self):
+                    try:
+                        self.__emitter.emit(SIGNAL("scanStarted"))
+
+                        inquirer = bluetooth.WCInquirer()
+                        addresses = []
+
+                        inquirer.start_inquiry()
+                        while inquirer.inquiry_in_progress:
+                            inquirer.read_msg()
+
+                            if inquirer.recently_discovered:
+                                bdaddr, devclass, bdname, connected = inquirer.recently_discovered[-1]
+                                if bdaddr not in addresses:
+                                    addresses.append(bdaddr)
+                                    # convert string to int
+                                    if len(devclass) == 3:
+                                        hex = "%02X%02X%02X" % (ord(devclass[0]), ord(devclass[1]), ord(devclass[2]))
+                                        devclass = int(hex, 16)
+                                    else:
+                                        devclass = 0
+                                    self.__emitter.emit(SIGNAL("foundDevice"),  bdaddr,  bdname,  devclass)
+
+                        self.__emitter.emit(SIGNAL("scanCompleted"))
+                    except Exception,  msg:
+                        self.__emitter.emit(SIGNAL("scanFailed"), msg[0])
+
+        else:
+            # Asynchronous mode isn't implemented
+            # There is no way to get the bluetooth class currently
+            # We would need to pass the flag LUP_RETURN_TYPE (Retrieves the type as lpServiceClassId)
+            # to _msbt.c
+            class ScanDevices(QThread):
+                def __init__(self,  parent,  emitFromParent=False):
+                    super(ScanDevices,  self).__init__(parent)
+                    self.__parent = parent
+                    if emitFromParent:
+                        self.__emitter = self.__parent
+                    else:
+                        self.__emitter = self
+
+                def stop(self):
+                    # TODO: Kill this thread?
+                    pass
+
+                def run(self):
+                    try:
+                        self.__emitter.emit(SIGNAL("scanStarted"))
+
+                        devices = bluetooth.discover_devices( lookup_names=True,  lookup_class=True)
+                        for address,  name,  bdclass in devices:
+                            self.__parent.emit(SIGNAL("foundDevice"),  address,  name,  bdclass)
+                        self.__emitter.emit(SIGNAL("scanCompleted"))
+                    except IOError,  msg:
+                        self.__emitter.emit(SIGNAL("scanFailed"), msg[0])
 
-            def run(self):
-                try:
-                    self.__parent.emit(SIGNAL("scanStarted"))
-                    
-                    devices = dict(bluetooth.discover_devices( lookup_names=True))
-                    for address,  name in devices.iteritems():
-                        self.__parent.emit(SIGNAL("foundDevice"),  address,  name)
-                    self.__parent.emit(SIGNAL("scanCompleted"))
-                except IOError,  msg:
-                    self.__parent.emit(SIGNAL("scanFailed"), msg[0])
-        
     else:
         class ScanDevices(QThread,  bluetooth.DeviceDiscoverer):
-            def __init__(self,  parent):
+            def __init__(self,  parent,  emitFromParent=False):
                 QThread.__init__(self,  parent)
                 bluetooth.DeviceDiscoverer.__init__(self)
                 self.__parent = parent
-                
+
                 self.__running = False
 
+                if emitFromParent:
+                    self.__emitter = self.__parent
+                else:
+                    self.__emitter = self
+
             def run(self):
                 self.__running = True
-                
+
                 try:
                    self.find_devices(lookup_names = True, flush_cache = True)
                    self.process_inquiry()
-                except bluetooth.btcommon.BluetoothError, msg:
-                   self.__parent.emit(SIGNAL("scanFailed"), msg[0])
+                except (bluetooth.btcommon.BluetoothError, bluetooth._bluetooth.error), msg:
+                   self.__emitter.emit(SIGNAL("scanFailed"), msg[0])
                    self.__running = False
 
             def stop(self):
                 if self.__running:
                     self.cancel_inquiry()
-                    self.__parent.emit(SIGNAL("scanCompleted"))
+                    self.__emitter.emit(SIGNAL("scanCompleted"))
 
             def pre_inquiry(self):
-                self.__parent.emit(SIGNAL("scanStarted"))
+                self.__emitter.emit(SIGNAL("scanStarted"))
 
-            def device_discovered(self, address, device_class, name) :
-                self.__parent.emit(SIGNAL("foundDevice"),  address,  name)
+            def device_discovered(self, address, deviceClass, name) :
+                self.__emitter.emit(SIGNAL("foundDevice"),  address,  name,  deviceClass)
 
             def inquiry_complete(self):
-                self.__parent.emit(SIGNAL("scanCompleted"))
+                self.__emitter.emit(SIGNAL("scanCompleted"))
                 self.__running = False
 
     class Listen(QThread):
         def __init__(self,  parent):
             super(Listen,  self).__init__(parent)
             self.parent = parent
-        
+
         def run(self):
             self.connect(self.parent,  SIGNAL("requestSend(QString)"),  self.send)
-            
+
             try:
                 self.sock = bluetooth.BluetoothSocket( bluetooth.RFCOMM )
                 self.sock.connect((self.parent.device().bluetoothAddress(),  self.parent.device().port()))
@@ -595,11 +834,11 @@ if USE_PYBLUEZ:
                     recv = unicode(self.sock.recv(1000),  "utf8")
                     if recv:
                         data += recv
-                        
+
                         # Last part is either empty or the beginning of the next data segment
                         data = data.split(NUM_END_TEXT)
                         for part in data[:-1]:
-                            # Extract header and message 
+                            # Extract header and message
                             header = int(part.split(NUM_END_HEADER)[0])
                             message = unicode(part.split(NUM_END_HEADER)[1])
 
@@ -607,7 +846,7 @@ if USE_PYBLUEZ:
                             if header  == NUM_QUIT:
                                 return
                             self.emit(SIGNAL("dataAvailable"),  header,  message)
-                            
+
                         data = data[-1]
                     else:
                         # Got an empty string -> connection is closed
@@ -616,60 +855,66 @@ if USE_PYBLUEZ:
                 except Exception:
                     self.disconnect(self.parent,  SIGNAL("requestSend(QString)"),  self.send)
                     return
-        
+
         def send(self,  data):
             data = str(data.toAscii())
             try:
                 self.sock.send(data)
             except Exception:
                 self.disconnect(self.parent,  SIGNAL("requestSend(QString)"),  self.send)
-                self.emit(SIGNAL("lostConnection"))
+                #self.emit(SIGNAL("lostConnection"))
                 self.exit(1)
 
 elif USE_LIGHTBLUE:
     class ScanDevices(object):
-        def __init__(self,  parent):
+        def __init__(self,  parent,  emitFromParent=False):
             self.__parent = parent
             self.__scan = lightblue._lightblue._AsyncDeviceInquiry.alloc().init()
             self.__scan._setupdatenames(True)
             self.__scan.cb_started= self.scanStarted
             self.__scan.cb_completed = self.scanCompleted
             self.__scan.cb_founddevice = self.scanFoundDevice
-        
+
+            if emitFromParent:
+                self.__emitter = self.__parent
+            else:
+                self.__emitter = self
+
         def start(self):
             ret = self.__scan.start()
             if ret != lightblue._macutil.kIOReturnSuccess:
-                self.__parent.emit(SIGNAL("scanFailed"), str(ret))
- 
+                self.__emitter.emit(SIGNAL("scanFailed"), str(ret))
+
         def stop(self):
             self.__scan.stop()
 
         def scanStarted(self):
-            self.__parent.emit(SIGNAL("scanStarted"))
+            self.__emitter.emit(SIGNAL("scanStarted"))
 
         def scanFoundDevice(self,  device):
             name = device.getName()
             if not name:
                 name = lightblue.finddevicename(device.getAddressString())
             addr = device.getAddressString().replace("-", ":").encode('ascii').upper()
-            self.__parent.emit(SIGNAL("foundDevice"),  addr,  name)
-        
+            deviceClass = device.getClassOfDevice()
+            self.__emitter.emit(SIGNAL("foundDevice"),  addr,  name, deviceClass)
+
         def scanCompleted(self,  err, aborted):
             if err:
-                self.__parent.emit(SIGNAL("scanFailed"), str(err))
+                self.__emitter.emit(SIGNAL("scanFailed"), str(err))
             else:
-                self.__parent.emit(SIGNAL("scanCompleted"))
+                self.__emitter.emit(SIGNAL("scanCompleted"))
 
     class Listen(QObject):
         def __init__(self,  parent):
             super(Listen,  self).__init__(parent)
             self.parent = parent
-            
+
             self.connect(self.parent,  SIGNAL("requestSend(QString)"),  self.send)
-            
+
         def start(self):
             self.__data = u""
-            
+
             self.sock = lightblue.socket( lightblue.RFCOMM )
             self.sock.setblocking(False)
             try:
@@ -677,7 +922,7 @@ elif USE_LIGHTBLUE:
                 self.startTimer(60)
             except socket.error,  msg:
                 self.parent.emit(SIGNAL("connectionFailed"),  msg[0],  msg[1])
-        
+
         def timerEvent(self,  event):
             try:
                 recv = unicode(self.sock.recv(10000),  "utf8")
@@ -688,12 +933,12 @@ elif USE_LIGHTBLUE:
             except Exception:
                 self.parent._lostConnection()
                 return
-            
+
             self.__data += recv
             # Last part is either empty or the beginning of the next data segment
             self.__data = self.__data.split(NUM_END_TEXT)
             for part in self.__data[:-1]:
-                # Extract header and message 
+                # Extract header and message
                 header = int(part.split(NUM_END_HEADER)[0])
                 message = unicode(part.split(NUM_END_HEADER)[1])
 
@@ -702,7 +947,7 @@ elif USE_LIGHTBLUE:
                     self.parent._lostConnection()
                     return
                 self.emit(SIGNAL("dataAvailable"),  header,  message)
-                
+
             self.__data = self.__data[-1]
 
         def send(self,  data):
