@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2008 - 2009 Lukas Hetzenecker <LuHe@gmx.at>
+# Copyright (c) 2008 - 2010 Lukas Hetzenecker <LuHe@gmx.at>
 
 NUM_CONNECTED = 100
 NUM_HELLO_REQUEST = 110
@@ -46,7 +46,24 @@ NUM_MESSAGE_REPLY_END = 353
 NUM_MESSAGE_REQUEST_UNREAD = 370
 NUM_MESSAGE_REPLY_UNREAD = 371
 
-NUM_INCOMING_CALL = 390
+NUM_CALENDAR_REQUEST_HASH_ALL = 380
+#NUM_CALENDAR_REQUEST_HASH_SINGLE = 381
+NUM_CALENDAR_REQUEST_ENTRY = 382
+NUM_CALENDAR_REQUEST_ENTRIES_ALL = 383
+NUM_CALENDAR_REPLY_HASH_ALL= 384
+#NUM_CALENDAR_REPLY_HASH_SINGLE_START= 385
+#NUM_CALENDAR_REPLY_HASH_SINGLE_LINE= 386
+#NUM_CALENDAR_REPLY_HASH_SINGLE_END= 387
+NUM_CALENDAR_REPLY_ENTRIES_START = 388
+NUM_CALENDAR_REPLY_ENTRY = 389
+NUM_CALENDAR_REPLY_ENTRIES_END = 390
+NUM_CALENDAR_ENTRY_ADD = 395
+NUM_CALENDAR_ENTRY_ADD_REPLY = 396
+NUM_CALENDAR_ENTRY_DELETE = 397
+NUM_CALENDAR_ENTRY_CHANGE = 398
+NUM_CALENDAR_ENTRY_CHANGE_REPLY_TIME = 399
+
+NUM_INCOMING_CALL = 400
 
 NUM_DEBUG = 999
 
@@ -54,4 +71,4 @@ NUM_END_HEADER = chr(0x02)   # Start of Text
 NUM_SEPERATOR = chr(0x1E)   # Record Separator
 NUM_END_TEXT = chr(0x03)   # End of Text
 
-PROTOCOL_VERSION = 1.4
+PROTOCOL_VERSION = 1.5
