@@ -6,7 +6,7 @@ echo "# Created: " `date -R` >> series60-remote.pro
 echo "# WARNING! All changes made in this file will be lost!" >> series60-remote.pro
 echo "" >> series60-remote.pro
 
-FILES=$(find devices/ lib/ ui/ widget/ window/ -name "*.py")
+FILES=$(find devices/ lib/ widget/ window/ -name "*.py")
 for file in $FILES
 do
    echo "SOURCES      +=  $file" >> series60-remote.pro
