@@ -1,9 +1,10 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2008 - 2009 Lukas Hetzenecker <LuHe@gmx.at>
+# Copyright (c) 2008 - 2010 Lukas Hetzenecker <LuHe@gmx.at>
 
 import hashlib
 import re
+import copy
 from PyQt4.QtCore import *
 from PyQt4.QtSql import *
 from lib.classes import *
@@ -12,6 +13,7 @@ THUMBNAIL = "thumbnail_image"
 
 # For hash functions
 CONTACT_SEP = chr(0x1F) # Unit Separator
+ENTRY_SEP = chr(0x1F) # Unit Separator
 FIELD_SEP = chr(0x1E) # Record Separator
 INFO_SEP = chr(0x1D) # Group Separator
 
@@ -157,6 +159,7 @@ class Database(QObject):
         deviceId INT UNSIGNED PRIMARY KEY,
         name VARCHAR(255) NOT NULL,
         address VARCHAR(255) NOT NULL,
+        deviceClass INT UNSIGNED,
         display  VARCHAR(255),
         imei INT UNSIGNED,
         model VARCHAR(255),
@@ -183,6 +186,33 @@ class Database(QObject):
         )
         """)
 
+        if not ret:
+            return False
+
+        ret = self.__query("""CREATE TABLE calendar (
+        entryId INT UNSIGNED NOT NULL PRIMARY KEY,
+        entryIdOnPhone INT,
+        device INT UNSIGNED NOT NULL,
+        type VARCHAR(255) NOT NULL,
+        content TEXT NOT NULL,
+        location TEXT,
+        startTime DATETIME,
+        endTime DATETIME,
+        lastModifiedTime DATETIME,
+        replication VARCHAR(255) NOT NULL,
+        alarmTime DATETIME,
+        priority INT UNSIGNED NOT NULL,
+        repeatType VARCHAR(255),
+        repeatDays VARCHAR(255),
+        repeatExceptions VARCHAR(255),
+        repeatStartTime DATETIME,
+        repeatEndTime DATETIME,
+        repeatInterval INT UNSIGNED,
+        crossedOut SMALLINT UNSIGNED NOT NULL DEFAULT 0 CHECK(crossedOut IN (0, 1)),
+        crossOutTime DATETIME
+        )
+        """)
+
         if not ret:
             return False
 
@@ -224,13 +254,8 @@ class Database(QObject):
         ret = myQuery.exec_()
         if self.main.sqlDebug:
             log = str(myQuery.lastQuery())
-            length = len(myQuery.boundValues())
-            for i in xrange(length):
-                value = unicode(myQuery.boundValue(i).toString())
-                if i+1 == length:
-                    log = re.sub( r":(\w|\d)*\)(.*)", value + r")\2", log, 1)
-                else:
-                    log = re.sub( r":(\w|\d)*,(.*)", value + r",\2", log, 1)
+            for key,  value in myQuery.boundValues().iteritems():
+                log = re.sub( str(key) + r"([\s|)|,])", unicode(value.toString()) + r"\1", log, 1)
             self.log.log(9,  QString("Execute query: %1").arg(log))
         return ret
 
@@ -313,9 +338,9 @@ class Database(QObject):
             return "strftime('%H', time)"
 
     def error(self):
-        err = str(self.__lastError).strip()
+        err = unicode(self.__lastError).strip()
         if not err:
-            err = self.tr("Unknown error")
+            err = unicode(self.tr("Unknown error"))
 
         return err
 
@@ -356,15 +381,15 @@ class Database(QObject):
         contacts = self.__contactDict()
 
         hash = unicode()
-        
+
         for key in sorted(contacts):
             hash += str(key)
-            hash += FIELD_SEP 
+            hash += FIELD_SEP
             for type,  location, value in contacts[key]:
                 hash += type + INFO_SEP + location + INFO_SEP + value
                 hash += FIELD_SEP
             hash += CONTACT_SEP
-        
+
         hash = hash.encode("utf8")
         hash = hashlib.md5(hash).hexdigest()
         return hash
@@ -372,7 +397,7 @@ class Database(QObject):
     def contactHashSingle(self):
         contacts = self.__contactDict()
         hashes = dict()
-        
+
         for key in sorted(contacts):
             hash = unicode()
             for type,  location, value in contacts[key]:
@@ -381,7 +406,7 @@ class Database(QObject):
             hash = hash.encode("utf8")
             hash = hashlib.md5(hash).hexdigest()
             hashes[key] = hash
-        
+
         return hashes
 
     def contactAdd(self,  contact):
@@ -426,6 +451,7 @@ class Database(QObject):
         query.bindValue(":value",  QVariant(value))
         self.__queryExec(query)
 
+        # TODO: Where should the id come from?!?
         return id
 
     def contactRemove(self,  contactIdOnPhone):
@@ -535,7 +561,6 @@ class Database(QObject):
                 contact.addValue(ContactField(type,  location),  value)
             yield contact
 
-
     def contactsWithMessages(self):
         ID,  NAME = range(2)
         query = self.__queryReturn("SELECT DISTINCT m.contact, c.name from messages m JOIN contacts c ON m.contact=c.contactId ORDER BY c.name ASC")
@@ -590,6 +615,231 @@ class Database(QObject):
     def contactUnfavoriteAll(self):
         self.__query("UPDATE contacts SET favoriteContact=0")
 
+    def __calendarNextId(self):
+        query = self.__queryReturn("SELECT MAX(entryId) FROM calendar")
+        if query and query.last():
+            return query.value(0).toInt()[0] +1
+        else:
+            return 1
+
+    def calendarEntriesCount(self):
+        query = self.__queryReturn("SELECT COUNT(entryIdOnPhone) FROM calendar")
+
+        if not query:
+            return 0
+
+        if query.last():
+            return query.value(0).toInt()[0]
+        else:
+            return 0
+
+    def calendarEntries(self,  start=None,  end=None,  repeatedEntries=False):
+        duration = list()
+        repeations = list()
+
+        # TODO: handle repeated entries...
+        if start:
+            duration.append("endTime >= '" + start.toString(Qt.ISODate) + "'")
+            if repeatedEntries:
+                repeations.append("(repeatEndTime >= '" + start.toString(Qt.ISODate) + "' OR repeatEndTime IS NULL)")
+        if end:
+            duration.append("startTime <= '" + end.toString(Qt.ISODate) + "'")
+            if repeatedEntries:
+                repeations.append("repeatStartTime <= '" + end.toString(Qt.ISODate) + "'")
+
+        where = ""
+        if duration or repeations:
+            where += " WHERE"
+            if duration:
+                where += '(' + self.__make("",  duration) + ' )'
+            if duration and repeations:
+                where += " OR "
+            if repeations:
+                where += '(' + self.__make("",  repeations) + ' )'
+
+        ID,  IDONPHONE,  DEVICE,  TYPE,  CONTENT,  LOCATION,  STARTTIME,  ENDTIME,  LASTMODIFIEDTIME,  REPLICATION,  ALARMTIME,  PRIORITY, \
+        REPEATTYPE,  REPEATDAYS,  REPEATEXCEPTIONS,  REPEATSTARTTIME,  REPEATENDTIME,  REPEATINTERVAL,  CROSSEDOUT,  CROSSOUTTIME = range(20)
+        query = """SELECT entryId, entryIdOnPhone, device, type, content, location, startTime, endTime, lastModifiedTime, replication, alarmTime,
+                         priority, repeatType, repeatDays, repeatExceptions, repeatStartTime, repeatEndTime, repeatInterval, crossedOut, crossOutTime
+                         FROM calendar """ + where + " ORDER BY entryIdOnPhone"
+        query = self.__queryReturn(query)
+
+        while query and query.next():
+            entry = CalendarEntry()
+            entry.setId(int(query.value(ID).toInt()[0]))
+            entry.setIdOnPhone(int(query.value(IDONPHONE).toInt()[0]))
+            entry.setDevice(self.devices(deviceId=int(query.value(DEVICE).toInt()[0])).next())
+            entry.setType(str(query.value(TYPE).toString()))
+            entry.setContent(unicode(query.value(CONTENT).toString()))
+            entry.setLocation(unicode(query.value(LOCATION).toString()))
+            entry.setStartTime(query.value(STARTTIME).toDateTime())
+            entry.setEndTime(query.value(ENDTIME).toDateTime())
+            entry.setLastModified(query.value(LASTMODIFIEDTIME).toDateTime())
+            entry.setReplication(str(query.value(REPLICATION).toString()))
+            entry.setAlarm(query.value(ALARMTIME).toDateTime())
+            entry.setPriority(int(query.value(PRIORITY).toInt()[0]))
+            entry.setRepeatType(str(query.value(REPEATTYPE).toString()))
+            entry.setRepeatDays(str(query.value(REPEATDAYS).toString()))
+            entry.setRepeatExceptions(str(query.value(REPEATEXCEPTIONS).toString()))
+            entry.setRepeatStart(query.value(REPEATSTARTTIME).toDateTime())
+            entry.setRepeatEnd(query.value(REPEATENDTIME).toDateTime())
+            entry.setRepeatInterval(int(query.value(REPEATINTERVAL).toInt()[0]))
+            entry.setCrossedOut(bool(int(query.value(CROSSEDOUT).toInt()[0])))
+            entry.setCrossOutTime(query.value(CROSSOUTTIME).toDateTime())
+
+            yield entry
+
+            if entry.repeatType() and repeatedEntries:
+                # This is a repeated entry...
+                #print "repeated entry:",  repr(entry)
+                date = start
+                while True:
+                    date = entry.next(date)
+
+                    if date > end or date is None:
+                        break
+                    #print "date is",  date
+                    repeatedEntry = copy.deepcopy(entry)
+                    repeatedEntry.setRealStartTime(entry.startTime())
+                    repeatedEntry.setRealEndTime(entry.endTime())
+                    repeatedEntry.setStartTime(date)
+                    repeatedEntry.setEndTime(date)
+                    yield repeatedEntry
+                #print "-----------------------------\n\n"
+
+    def calendarHash(self):
+        hash = unicode()
+
+        for entry in self.calendarEntries():
+            hash += str(entry.idOnPhone())
+            hash += FIELD_SEP
+
+            # Format the entry values....
+            hash += entry.type() + FIELD_SEP + entry.content() + FIELD_SEP + entry.location() + FIELD_SEP
+            if entry.startTime():
+                hash += str(entry.startTime().toTime_t())
+            hash += FIELD_SEP
+
+            if entry.endTime():
+                hash += str(entry.endTime().toTime_t())
+            hash += FIELD_SEP
+
+            hash += str(entry.lastModified().toTime_t()) + FIELD_SEP
+            hash += entry.replication() + FIELD_SEP
+
+            if entry.alarm():
+                hash += str(entry.alarm().toTime_t())
+            hash += FIELD_SEP
+
+            hash += str(entry.priority()) + FIELD_SEP
+
+            hash += entry.repeatType() + FIELD_SEP + entry.repeatDays() + FIELD_SEP + entry.repeatExceptions() + FIELD_SEP
+            if entry.repeatStart():
+                hash += str(entry.repeatStart().toTime_t())
+            hash += FIELD_SEP
+            if entry.repeatEnd():
+                hash += str(entry.repeatEnd().toTime_t())
+            hash += FIELD_SEP
+            if entry.repeatInterval():
+                hash += str(entry.repeatInterval())
+            hash += FIELD_SEP
+
+            if entry.type() == "todo":
+                hash += str(int(entry.crossedOut())) + FIELD_SEP
+
+                if entry.crossOutTime():
+                    hash += str(entry.crossOutTime().toTime_t())
+                hash += FIELD_SEP
+
+
+            hash += FIELD_SEP
+            hash += ENTRY_SEP
+
+        #print "PC:::" + hash.replace(FIELD_SEP,  ";").replace(ENTRY_SEP,  "\n")
+
+        hash = hash.encode("utf8")
+        hash = hashlib.md5(hash).hexdigest()
+        return hash
+
+    def calendarIds(self):
+        ret = list()
+        query = self.__queryReturn("SELECT entryIdOnPhone FROM calendar")
+        while query and query.next():
+            id = int(query.value(0).toInt()[0])
+            ret.append(id)
+        return ret
+
+    def __calendarEntryBind(self,  entry,  query):
+        if entry.id():
+            id = entry.id()
+        else:
+            id = self.__calendarNextId()
+
+        query.bindValue(":entryId",  QVariant(id))
+        query.bindValue(":entryIdOnPhone",  QVariant(entry.idOnPhone()))
+        query.bindValue(":device",  QVariant(entry.device().id()))
+        query.bindValue(":type",  QVariant(entry.type()))
+        query.bindValue(":content",  QVariant(entry.content()))
+        query.bindValue(":location",  QVariant(entry.location()))
+        query.bindValue(":startTime",  QVariant(entry.realStartTime()))
+        query.bindValue(":endTime",  QVariant(entry.realEndTime()))
+        query.bindValue(":lastModifiedTime",  QVariant(entry.lastModified()))
+        query.bindValue(":replication",  QVariant(entry.replication()))
+        query.bindValue(":alarmTime",  QVariant(entry.alarm()))
+        query.bindValue(":priority",  QVariant(entry.priority()))
+        query.bindValue(":repeatType",  QVariant(entry.repeatType()))
+        query.bindValue(":repeatDays",  QVariant(entry.repeatDays()))
+        query.bindValue(":repeatExceptions",  QVariant(entry.repeatExceptions())) if entry.repeatExceptions() else QVariant()
+        query.bindValue(":repeatStartTime",  QVariant(entry.repeatStart()))
+        query.bindValue(":repeatEndTime",  QVariant(entry.repeatEnd())) if entry.repeatEnd() and entry.repeatEnd().isValid() else QVariant()
+        query.bindValue(":repeatInterval",  QVariant(entry.repeatInterval()))
+        query.bindValue(":crossedOut",  QVariant(int(entry.crossedOut())))
+        query.bindValue(":crossOutTime",  QVariant(entry.crossOutTime()))
+
+    def calendarEntryAdd(self,  entry):
+        query = QSqlQuery()
+        query.prepare("""INSERT INTO calendar (entryId, entryIdOnPhone, device, type, content, location, startTime, endTime, lastModifiedTime, replication, alarmTime,
+        priority, repeatType, repeatDays, repeatExceptions, repeatStartTime, repeatEndTime, repeatInterval, crossedOut, crossOutTime)
+        VALUES (:entryId, :entryIdOnPhone, :device, :type, :content, :location, :startTime, :endTime, :lastModifiedTime, :replication, :alarmTime,
+        :priority, :repeatType, :repeatDays, :repeatExceptions, :repeatStartTime, :repeatEndTime, :repeatInterval, :crossedOut, :crossOutTime)""")
+
+        self.__calendarEntryBind(entry,  query)
+        return self.__queryExec(query)
+
+    def calendarEntryUpdate(self,  entry):
+        query = QSqlQuery()
+
+        if entry.id():
+            # This is used when the entry is updated on the PC with the calendar_edit dialog
+            where = "WHERE entryId=" + str(entry.id())
+        elif entry.idOnPhone():
+            # This is needed during the synchronisation
+            where = "WHERE entryIdOnPhone=" + str(entry.idOnPhone())
+
+        query.prepare("""UPDATE calendar SET device=:device, type=:type, content=:content, location=:location, startTime=:startTime, endTime=:endTime,
+        lastModifiedTime=:lastModifiedTime, replication=:replication, alarmTime=:alarmTime, priority=:priority, repeatType=:repeatType, repeatDays=:repeatDays,
+        repeatExceptions=:repeatExceptions, repeatStartTime=:repeatStartTime, repeatEndTime=:repeatEndTime, repeatInterval=:repeatInterval, crossedOut=:crossedOut,
+        crossOutTime=:crossOutTime """ + where)
+
+        self.__calendarEntryBind(entry,  query)
+        return self.__queryExec(query)
+
+    def calendarEntryUpdateLastModifiedTime(self,  entryIdOnPhone,  time):
+        query = QSqlQuery()
+        query.prepare("UPDATE calendar SET lastModifiedTime=:lastModifiedTime WHERE entryIdOnPhone=" + str(entryIdOnPhone))
+        query.bindValue(":lastModifiedTime",  QVariant(time))
+        return self.__queryExec(query)
+
+    def calendarEntryRemove(self,  entry):
+        return self.__query("DELETE FROM calendar WHERE entryId='" + str(entry.id()) + "'")
+
+    def calendarEntryRemoveByIdOnPhone(self,  id):
+        return self.__query("DELETE FROM calendar WHERE entryIdOnPhone='" + str(id) + "'")
+
+    def calendarEntriesRemove(self,  list):
+        for entry in list:
+            self.calendarEntryRemove(entry)
+
     def __deviceNextId(self):
         query = self.__queryReturn("SELECT MAX(deviceId) FROM devices")
         if query and query.last():
@@ -624,11 +874,12 @@ class Database(QObject):
         id = self.__deviceNextId()
 
         query = QSqlQuery()
-        query.prepare("""INSERT INTO devices (deviceId, name, address, showDevice)
-        VALUES (:deviceId, :name, :address, 1)""")
+        query.prepare("""INSERT INTO devices (deviceId, name, address, deviceClass, showDevice)
+        VALUES (:deviceId, :name, :address, :deviceClass, 1)""")
         query.bindValue(":deviceId",  QVariant(id))
         query.bindValue(":name",  QVariant(device.name()) )
         query.bindValue(":address",  QVariant(str(device.bluetoothAddress())))
+        query.bindValue(":deviceClass",  QVariant(device.deviceClass()))
         self.__queryExec(query)
 
         return id
@@ -648,7 +899,12 @@ class Database(QObject):
         return self.__query("UPDATE devices SET showDevice=0")
 
     def deviceShow(self,  device):
-        return self.__query("UPDATE devices SET showDevice=1 WHERE address='" + str(device.bluetoothAddress()) + "'")
+        if not self.__query("UPDATE devices SET showDevice=1 WHERE address='" + str(device.bluetoothAddress()) + "'"):
+            return False
+
+        # TODO! Only needed when upgrading from old database version
+        return self.__query("UPDATE devices SET deviceClass='" + str(device.deviceClass()) + "'"
+                                        " WHERE address='" + str(device.bluetoothAddress()) + "'")
 
     def deviceCheckValues(self,  device):
         ID,  NAME,  ADDRESS,  DISPLAY,  IMEI,  MODEL,  S60,  RAM,  ROM = range(9)
@@ -662,9 +918,9 @@ class Database(QObject):
                 and query.value(S60).toBool() and query.value(RAM).toBool() and query.value(ROM).toBool())
 
     def devices(self,  deviceId=None,  bluetoothAddress=None):
-        ID,  NAME,  ADDRESS,  DISPLAY,  IMEI,  MODEL,  S60,  RAM,  ROM = range(9)
+        ID,  NAME,  ADDRESS,  DEVICECLASS,  DISPLAY,  IMEI,  MODEL,  S60,  RAM,  ROM = range(10)
 
-        query = """SELECT deviceId, name, address, display, imei, model, s60_version, total_ram, total_rom
+        query = """SELECT deviceId, name, address, deviceClass, display, imei, model, s60_version, total_ram, total_rom
         from devices WHERE showDevice=1"""
 
         if deviceId:
@@ -679,6 +935,7 @@ class Database(QObject):
             device.setId(int(query.value(ID).toInt()[0]))
             device.setName(unicode(query.value(NAME).toString()))
             device.setBluetoothAddress(str(query.value(ADDRESS).toString()))
+            device.setDeviceClass(int(query.value(DEVICECLASS).toInt()[0]))
 
             if self.deviceCheckValues(device):
                 device.addValue("display",  str(query.value(DISPLAY).toString()))
@@ -784,14 +1041,14 @@ class Database(QObject):
             msg.setContact(self.contacts(contactId=int(query.value(CONTACT).toInt()[0])).next())
             msg.setMessage(unicode(query.value(MESSAGE).toString()))
             ret.insert(0,  msg)
-        
+
         return ret
 
     def messageCount(self,  contactFilter=None):
         where = ""
         if contactFilter:
             where = "WHERE contact IN (" + ",".join([str(contact.id()) for contact in contactFilter]) + ")"
-        
+
         query = self.__queryReturn("SELECT COUNT(messageId) FROM messages " + where)
 
         if not query:
@@ -802,7 +1059,7 @@ class Database(QObject):
         else:
             return 0
 
-    def messages(self,  year=None,  month=None,  day=None,  contact=None,  filter=None,  messageId=None,  
+    def messages(self,  year=None,  month=None,  day=None,  contact=None,  filter=None,  messageId=None,
                  messageIdOnPhone=None,  order="DESC",  contactFilter=None):
         query = "SELECT messageId, messageIdOnPhone, type, time, contact, message FROM messages "
         conditions = list()
@@ -1075,6 +1332,7 @@ class Update(object):
         self.versions = list()
         self.versions.append(( (0,  2),  (0,  3),  self.update_02_to_03 ))
         self.versions.append(( (0,  3),  (0,  4),  self.update_03_to_04 ))
+        self.versions.append(( (0,  4),  (0,  5),  self.update_04_to_05 ))
 
     def update(self,  fromVersion,  toVersion):
         froms = ( i[0] for i in self.versions )
@@ -1254,3 +1512,40 @@ class Update(object):
             return False
 
         return True
+
+    def update_04_to_05(self):
+        query = QSqlQuery(self.database)
+        ret = query.exec_("""ALTER TABLE devices
+        ADD COLUMN deviceClass INT UNSIGNED""")
+
+        if not ret:
+            return False
+
+        ret = query.exec_("""CREATE TABLE calendar (
+        entryId INT UNSIGNED NOT NULL PRIMARY KEY,
+        entryIdOnPhone INT,
+        device INT UNSIGNED NOT NULL,
+        type VARCHAR(255) NOT NULL,
+        content TEXT NOT NULL,
+        location TEXT,
+        startTime DATETIME,
+        endTime DATETIME,
+        lastModifiedTime DATETIME,
+        replication VARCHAR(255) NOT NULL,
+        alarmTime DATETIME,
+        priority INT UNSIGNED NOT NULL,
+        repeatType VARCHAR(255),
+        repeatDays VARCHAR(255),
+        repeatExceptions VARCHAR(255),
+        repeatStartTime DATETIME,
+        repeatEndTime DATETIME,
+        repeatInterval INT UNSIGNED,
+        crossedOut SMALLINT UNSIGNED NOT NULL DEFAULT 0 CHECK(crossedOut IN (0, 1)),
+        crossOutTime DATETIME
+        )
+        """)
+
+        if not ret:
+            return False
+
+        return True
