@@ -11,7 +11,7 @@ class QtStreamHandler(logging.Handler):
         logging.Handler.__init__(self)
         self.parent = parent
         self.main = main
-        
+
         self.textWidget = parent
         self.formater = logging.Formatter("%(message)s")
 
@@ -20,13 +20,13 @@ class QtStreamHandler(logging.Handler):
 
     def createLock(self):
         self.mutex = QMutex()
-    
+
     def acquire(self):
         self.mutex.lock()
-    
+
     def release(self):
         self.mutex.unlock()
-    
+
     def emit(self,record):
         self.textWidget.appendPlainText(self.formater.format(record))
         self.textWidget.moveCursor(QTextCursor.StartOfLine)
@@ -37,18 +37,21 @@ class QtOutput(object):
         self.textWidget = parent
         self.out = out
         self.color = color
-    
+
     def write(self, m):
         self.textWidget.moveCursor(QTextCursor.End)
-        
+
         if self.color:
             tc = self.textWidget.textColor()
             self.textWidget.setTextColor(self.color)
-        
+
         self.textWidget.insertPlainText( m )
-        
+
         if self.color:
             self.textWidget.setTextColor(tc)
-        
+
         if self.out:
-            self.out.write(m)
+            if isinstance(m,  unicode):
+                self.out.write(m.encode("utf8"))
+            else:
+                self.out.write(m)
