@@ -6,11 +6,11 @@ import sys
 import re
 import base64
 import copy
-import xml.dom.minidom
+import distutils.version
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 import window.contacts_edit
-import window.chat
+import window.contacts_import
 import window.settings
 import window.history
 import window.statistics
@@ -28,6 +28,7 @@ import ui.ui_connection_failed
 import ui.ui_connection_version_mismatch
 import ui.ui_connection_update_version
 import ui.resource_rc
+import lib.update_checker
 import lib.favorites
 import lib.obex_handler
 import lib.obex_scheduler
@@ -54,7 +55,7 @@ class MainWindow(QMainWindow,  ui.ui_main.Ui_MainWindow):
         self.refreshTimer.setInterval(600000)
 
         self.setupUi(self)
-        
+
         # Favorites menu
         self.contactMenu = self.menu_Contacts
         self.favMenu = lib.favorites.FavoriteMenu(self.contactMenu,  main)
@@ -65,6 +66,12 @@ class MainWindow(QMainWindow,  ui.ui_main.Ui_MainWindow):
         pal.setColor(QPalette.Highlight,  QColor(38,  136,  240))
         self.listWidget.setPalette(pal)
 
+        # Add menu to "Import contacts" button
+        self.importMenu = QMenu(self)
+        self.importVcardAction = self.importMenu.addAction(QIcon(":/text-x-vcard"),  self.tr("Import &Vcard file..."))
+        self.importLdifAction = self.importMenu.addAction(QIcon(":/text-x-ldif"),  self.tr("Import &LDIF file..."))
+        self.contactsImportButton.setMenu(self.importMenu)
+
         # Restore size, position and splitter states from previous saved value
         windowSize = self.settings.setting("windows/main/size")
         windowPosition = self.settings.setting("windows/main/position")
@@ -83,17 +90,29 @@ class MainWindow(QMainWindow,  ui.ui_main.Ui_MainWindow):
         if not contactsSplitter.isNull():
             self.contactsSplitter.restoreState(contactsSplitter)
 
-        self.recipients = list()
         self.newMessagesComplete = False
         self.queueMessages = 0
         self.fillTypeBox = True
+        self.connectionAttemptByUser = True
+        self.connectionClosedByUser = False
+        self.deviceScanner = self.connection.scanner()
+        self.automaticConnectionTimer = QTimer()
+
+        self.emptyPixmap = QPixmap(16,  16)
+        self.emptyPixmap.fill(Qt.transparent)
+        self.emptyIcon = QIcon(self.emptyPixmap)
+        self.contactIsRecipientIcon = QIcon(":/dialog-apply")
 
         self.fileWidget.updateActions()
+
+        # Load the contacts and devices when the event loop is started and all other events are handled
+        # This results in a faster startup (saved ~274ms)
         self.loadSettings()
         self.showFavorites()
+        QTimer.singleShot(0,  self.loadUpdateChecker)
+        QTimer.singleShot(0,  lambda : self.loadAutomaticConnection(True))
+
         self.adjustSize()
-        self.checkSendButton()
-        self.checkEditContactButton()
 
         self.connect(self.main,  SIGNAL("favoriteListChanged"),  self.showFavorites)
         self.connect(self.main,  SIGNAL("updateContact"),  self.updateContact)
@@ -121,21 +140,24 @@ class MainWindow(QMainWindow,  ui.ui_main.Ui_MainWindow):
         self.connect(self.disconnectButton, SIGNAL("clicked()"),  self.closeConnection)
         self.connect(self.cancelButton, SIGNAL("clicked()"),  self.closeConnection)
         self.connect(self.messageText,  SIGNAL("textChanged()"),  self.textChanged)
-        self.connect(self.toLine,  SIGNAL("textChanged(const QString &)"),  self.recipentChanged)
+        self.connect(self.toLine,  SIGNAL("textEdited(const QString &)"),  self.recipientChanged)
         self.connect(self.contactsTree,SIGNAL("customContextMenuRequested(QPoint)"),self.showCustomContextMenu)
-        self.connect(self.contactsTree,SIGNAL("itemDoubleClicked(QTreeWidgetItem *, int)"),self.insertContact)
+        self.connect(self.contactsTree,SIGNAL("itemActivated(QTreeWidgetItem *, int)"),self.contactClicked)
         self.connect(self.contactsList,SIGNAL("currentItemChanged(QListWidgetItem *, QListWidgetItem *)"),self.showContact)
         self.connect(self.contactsList,SIGNAL("currentItemChanged(QListWidgetItem *, QListWidgetItem *)"),
                                                                   self,  SLOT("checkEditContactButton()"))
-        self.connect(self.contactEditButton,SIGNAL("clicked()"),self.editContact)
-        self.connect(self.contactAddButton,SIGNAL("clicked()"),self.addContact)
+        self.connect(self.contactEditButton,SIGNAL("clicked()"), self.editContact)
+        self.connect(self.contactAddButton,SIGNAL("clicked()"), self.addContact)
+        self.connect(self.importVcardAction,SIGNAL("triggered()"), lambda : self.importContacts("vcard"))
+        self.connect(self.importLdifAction,SIGNAL("triggered()"), lambda : self.importContacts("ldif"))
         self.connect(self.contactsList,SIGNAL("customContextMenuRequested(QPoint)"),self.showContactListContextMenu)
         self.connect(self.searchLine,SIGNAL("textChanged(const QString &)"),self.showContacts)
         self.connect(self.searchLine_2,SIGNAL("textChanged(const QString &)"),self.showContacts)
         self.connect(self.typeBox,SIGNAL("currentIndexChanged(int)"),self.showContacts)
-        self.connect(self.connection, SIGNAL("connectionStateChanged"),  lambda: self.connectBar.setValue(self.connectBar.value() + 1))
+        self.connect(self.connection, SIGNAL("connectionStateChanged"),  lambda x: self.connectBar.setValue(x))
         self.connect(self.connection, SIGNAL("sysinfoCompleted"),  self.showSysinfo)
         self.connect(self.connection, SIGNAL("contactsCompleted"),  self.showContacts)
+        self.connect(self.connection, SIGNAL("contactsUpdated"),  self.showContacts)
         self.connect(self.connection, SIGNAL("connectionCompleted"),  self.connected)
         self.connect(self.connection, SIGNAL("connectionClosed"),  self.connectionClosed)
         self.connect(self.connection, SIGNAL("connectionAborted"),  self.connectionClosed)
@@ -145,15 +167,41 @@ class MainWindow(QMainWindow,  ui.ui_main.Ui_MainWindow):
         self.connect(self.connection, SIGNAL("messagesRequestComplete"),  self.newMessagesFinished)
         self.connect(self.connection, SIGNAL("messageSent"),  self.messageStateChanged)
         self.connect(self.connection, SIGNAL("messageQueued"),  self.messageStateChanged)
+        self.connect(self.automaticConnectionTimer,  SIGNAL("timeout()"),  self.automaticConnectionTimerFired)
+        self.connect(self.deviceScanner,  SIGNAL("scanStarted"),  self.automaticConnectionScanStarted)
+        self.connect(self.deviceScanner,  SIGNAL("foundDevice"),  self.automaticConnectionFoundDevice)
+        self.connect(self.deviceScanner,  SIGNAL("scanCompleted"),  self.automaticConnectionScanFinished)
+        self.connect(self.deviceScanner,  SIGNAL("scanFailed"),  self.automaticConnectionScanFinished)
         self.connect(self.settings,  SIGNAL("reloadSettings"),  self.loadSettings)
+        self.connect(self.settings,  SIGNAL("reloadSettings"),  self.loadAutomaticConnection)
 
-        self.show()
+        # Also update the icons in the summary tab when the connection state has changed
+
+        self.okPixmap = QIcon(":/dialog-apply").pixmap(16,  16)
+        self.loadingMovie = QMovie(":/loading-2",  parent=self)
+        self.loadingMovie.setScaledSize(QSize(20,  20))
+        self.loadingMovie.start()
+
+        self.connect(self.connection, SIGNAL("connectionEstablished"),  lambda : self.connectionStateLabel.setPixmap(self.okPixmap))
+        self.connect(self.connection, SIGNAL("connectionEstablished"),  lambda : self.sysinfoStateLabel.setMovie(self.loadingMovie))
+
+        self.connect(self.connection, SIGNAL("sysinfoCompleted"),  lambda : self.sysinfoStateLabel.setPixmap(self.okPixmap))
+        self.connect(self.connection, SIGNAL("sysinfoCompleted"),  lambda : self.contactStateLabel.setMovie(self.loadingMovie))
+
+        self.connect(self.connection, SIGNAL("contactsCompleted"),  lambda : self.contactStateLabel.setPixmap(self.okPixmap))
+        self.connect(self.connection, SIGNAL("contactsCompleted"),  lambda : self.calendarStateLabel.setMovie(self.loadingMovie))
+
+        self.connect(self.connection, SIGNAL("calendarCompleted"),  lambda : self.calendarStateLabel.setPixmap(self.okPixmap))
+
+
+        if not main.minimized:
+            self.show()
 
     def __str__(self):
         return "\"Main-Window\""
 
     def loadSettings(self):
-        self.addDevices()
+        self.updateDevices()
         if self.connection.connected():
             # Show the extended StackedWidget when there is an active connection
             # after reloading the settings
@@ -163,6 +211,42 @@ class MainWindow(QMainWindow,  ui.ui_main.Ui_MainWindow):
 
         self.messageText.setSendMessageOnReturn(self.settings.setting("general/sendMessageOnReturn"))
 
+        self.checkSendButton()
+        self.checkEditContactButton()
+
+    def loadUpdateChecker(self):
+        if self.settings.setting("updateCheck/enabled"):
+            lastCheck = self.settings.setting("updateCheck/lastCheck")
+            interval = self.settings.setting("updateCheck/interval")
+            if interval == 0:
+                return
+            if not lastCheck.isValid() or lastCheck.daysTo(QDate.currentDate()) >= interval:
+                self.updateChecker = lib.update_checker.UpdateChecker(self,  self.main)
+                self.connect(self.updateChecker,  SIGNAL("updateCheckFailed"),  self.updateCheckError)
+                self.connect(self.updateChecker,  SIGNAL("updateCheckNewVersion"),  self.updateCheckNewVersion)
+
+                self.updateChecker.updateCheck()
+            else:
+                lastVersion = self.settings.setting("updateCheck/lastVersion")
+                if not lastVersion:
+                    return
+                lastVersion = distutils.version.LooseVersion(lastVersion)
+
+                currentVersion = ".".join([str(i) for i in self.main.appVersion])
+                currentVersion = distutils.version.LooseVersion(currentVersion)
+
+                if lastVersion > currentVersion:
+                    self.updateCheckNewVersion(self.settings.setting("updateCheck/lastVersion"),  self.settings.setting("updateCheck/lastMessage"))
+
+    def loadAutomaticConnection(self,  firstStart=False):
+        enabled = self.settings.setting("general/automaticConnectionEnabled")
+        if enabled and not self.connection.connected():
+            interval = self.settings.setting("general/automaticConnectionInterval")
+            if firstStart:
+                self.automaticConnectionTimerFired()
+            self.automaticConnectionTimer.setInterval(interval * 1000)
+            self.automaticConnectionTimer.start()
+
     def showFavorites(self):
         self.contactMenu.clear()
         self.favMenu.menu(self.contactMenu)
@@ -199,31 +283,10 @@ class MainWindow(QMainWindow,  ui.ui_main.Ui_MainWindow):
                 scheduler = lib.obex_scheduler.ObexScheduler(handler)
                 self.fileWidget.setScheduler(scheduler)
 
-    def addDevices(self):
-        self.devicesBox.clear()
-        self.devicesBox_2.clear()
-        self.devicesBox_3.clear()
-        for device in self.database.devices():
-            self.devicesBox.addItem(device.name(),  QVariant(device))
-            self.devicesBox_2.addItem(device.name(),  QVariant(device))
-            self.devicesBox_3.addItem(device.name(),  QVariant(device))
-
-        last = self.settings.setting("bluetooth/lastName")
-        row = self.devicesBox.findText(last)
-        if not row == -1:
-            self.devicesBox.setCurrentIndex(row)
-            self.devicesBox_2.setCurrentIndex(row)
-            self.devicesBox_3.setCurrentIndex(row)
-
-        if self.devicesBox.count() == 0:
-            self.devicesBox.addItem(self.tr("No favorites"))
-            self.devicesBox_2.addItem(self.tr("No favorites"))
-            self.devicesBox_3.addItem(self.tr("No favorites"))
-            self.detailStack.setCurrentWidget(self.noDataWidget)
-        else:
+    def updateDevices(self):
+        device = self.devicesBox.currentDevice()
+        if not isinstance(device,  type(None)):
             try:
-                device = self.devicesBox.itemData(self.devicesBox.currentIndex()).toPyObject()
-
                 try:
                     totalRam = self.helper.pretty_filesize(device.value("total_ram"))
                 except:
@@ -241,32 +304,57 @@ class MainWindow(QMainWindow,  ui.ui_main.Ui_MainWindow):
                 self.displayLabel_3.setText(self.tr("%1 pixels").arg(device.value("display") ))
                 self.osLabel_3.setText(device.value("s60_version")) # TODO: append to modelLabel
                 self.detailStack.setCurrentWidget(self.simpleWidget)
-            except:
+            except ValueError:
+                # This happens when you were never connected to the device
+                # (e.g. when you start the application for the first time)
                 self.detailStack.setCurrentWidget(self.noDataWidget)
+        else:
+            self.detailStack.setCurrentWidget(self.noDataWidget)
 
-    def connectToDevice(self):
+    def __connectToDevice(self, device):
         if self.connection.connected():
             return
-        if self.devicesBox.currentText() == self.tr("No favorites"):
-            return
 
-        device = self.devicesBox.itemData(self.devicesBox.currentIndex()).toPyObject()
+        if isinstance(device,  type(None)):
+            return
 
         self.settings.setSetting("bluetooth/lastName",  device.name())
         port = self.settings.setting("bluetooth/port")
 
+        if self.scanningMovie.movie():
+            self.scanningMovie.movie().stop()
+        self.scanningMovie.setMovie(QMovie())
+        self.scanningMovie.setToolTip("")
+        self.automaticConnectionTimer.stop()
+
         # FIXME: Ugly hack
         device.setPort(port)
 
+        # Reset connection state icons
+        self.connectionStateLabel.setMovie(self.loadingMovie)
+        self.sysinfoStateLabel.clear()
+        self.contactStateLabel.clear()
+        self.calendarStateLabel.clear()
+
         self.log.info(QString("Connect to device %1 ( %2 on port %3 )").arg(device.name()).arg(device.bluetoothAddress()).arg(port))
         self.statusLabel.setText(self.tr("Establish connection!"))
         self.connectLabel.setText(self.tr("Connection establishment to: <b>%1</b>").arg(device.name()))
+        self.devicesBox.selectDevice(device)
         self.devicesBox.setEnabled(False)
         self.connectButton.setEnabled(False)
         self.establishConnectionStack.setCurrentWidget(self.establishConnectionWidget)
 
         self.connection.connectToDevice(device)
 
+    def connectToDevice(self):
+        device = self.devicesBox.currentDevice()
+        if isinstance(device,  type(None)):
+            return
+
+        self.connectionAttemptByUser = True
+        self.__connectToDevice(device)
+
+
     def showSysinfo(self):
         refreshDate = QDate.currentDate().toString("dd.MM.yyyy")
         refreshTime = QTime().currentTime().toString("hh:mm:ss")
@@ -309,7 +397,7 @@ class MainWindow(QMainWindow,  ui.ui_main.Ui_MainWindow):
         else:
             self.signalBar_2.setHidden(False)
             self.signalBar.setHidden(False)
-        
+
         self.refreshTimeLabel_2.setText(refreshDate + " " + refreshTime)
         self.modelLabel_2.setText(str(self.connection.device().value("model")))
         self.batteryLabel_2.setText(self.tr("%1% of 100%").arg(self.connection.device().value("battery")))
@@ -352,8 +440,6 @@ class MainWindow(QMainWindow,  ui.ui_main.Ui_MainWindow):
             self.typeBox.addItem(self.tr("All fields"),  QVariant("s60remote-all"))
             self.typeBox.insertSeparator(2)
 
-            self.typeBox.setCurrentIndex(0)
-
         search = self.searchLine.searchText()
         if not search:
             search = self.searchLine_2.searchText()
@@ -369,7 +455,7 @@ class MainWindow(QMainWindow,  ui.ui_main.Ui_MainWindow):
                         continue
                     if self.typeBox.findData(QVariant(field.type())) == -1:
                         self.typeBox.addItem(field.toString()[:-1],  QVariant(field.type()))
-            
+
             if search:
                 # Search for name
                 if searchField == "s60remote-name":
@@ -412,12 +498,14 @@ class MainWindow(QMainWindow,  ui.ui_main.Ui_MainWindow):
                         item = widget.SortedTreeWidgetItem.SortedTreeWidgetItem(self.contactsTree)
                         item.setData(0, Roles.ContactRole,  QVariant(contact))
                         item.setText(0, contact.name())
+                        item.setIcon(0,  self.contactIsRecipientIcon) if self.contactIsRecipient(contact) else item.setIcon(0,  self.emptyIcon)
                 else:
                     for number in contact.value("mobile_number"):
                         item = widget.SortedTreeWidgetItem.SortedTreeWidgetItem(self.contactsTree)
                         item.setData(0, Roles.ContactRole,  QVariant(contact))
                         item.setText(0, contact.name())
                         item.setText(1,  number)
+                        item.setIcon(0,  self.contactIsRecipientIcon) if self.contactIsRecipient(contact) else item.setIcon(0,  self.emptyIcon)
 
         if self.contactsList.currentRow() == -1 and self.contactsList.count() > 0:
             self.contactsList.setCurrentRow(0)
@@ -428,7 +516,6 @@ class MainWindow(QMainWindow,  ui.ui_main.Ui_MainWindow):
         self.contactsTree.resizeColumnToContents(1)
 
         if self.fillTypeBox:
-            self.typeBox.setCurrentIndex(0)
             self.fillTypeBox = False
 
     def updateContact(self,  contact):
@@ -470,6 +557,8 @@ class MainWindow(QMainWindow,  ui.ui_main.Ui_MainWindow):
     def connected(self):
         self.refreshTimer.start()
 
+        self.connectionClosedByUser = False
+
         self.connectionDate = QDate.currentDate().toString("dd.MM.yyyy")
         self.connectionTime = QTime().currentTime().toString("hh:mm:ss")
         self.connectionTimeLabel.setText(self.connectionDate + " " + self.connectionTime)
@@ -484,14 +573,43 @@ class MainWindow(QMainWindow,  ui.ui_main.Ui_MainWindow):
         self.checkEditContactButton()
         self.checkSendButton()
 
-    def insertContact(self,  item,  column):
-        name = item.text(0)
-        data = item.data(0,  Roles.ContactRole).toPyObject()
+    def contactClicked(self,  item,  column):
+        contact = item.data(0,  Roles.ContactRole).toPyObject()
+        phone = item.text(1) if not self.settings.setting("contacts/hideCellnumber") else None
+        if phone:
+            contact.addInternalValue("phone",  phone)
+        
+        if self.contactIsRecipient(contact):
+            self.takeContact(contact,  phone)
+            item.setIcon(0,  self.emptyIcon)
+        else:
+            self.insertContact(contact,  phone)
+            item.setIcon(0,  self.contactIsRecipientIcon)
+        
+        self.checkSendButton()
+    
+    def contactIsRecipient(self,  contact):
+        to = self.toLine.text()
+        
+        if not to:
+            return False
+        
+        hide = self.settings.setting("contacts/hideCellnumber")
+        for recipient in unicode(to).split(";"):
+            recipient = recipient.strip()
+            if hide:
+                if recipient == contact.name():
+                    return True
+            else:
+                if recipient == contact.name() + " (" + contact.internalValue("phone") + ")":
+                    return True
+        return False
+    
+    def insertContact(self,  contact,  phone):
+        name = contact.name()
 
-        if not self.settings.setting("contacts/hideCellnumber"):
-            phone = str(item.text(1))
+        if phone:
             name += " (" + phone + ")"
-            data.addInternalValue("phone",  phone)
 
         curName = unicode(self.toLine.text())
 
@@ -502,7 +620,19 @@ class MainWindow(QMainWindow,  ui.ui_main.Ui_MainWindow):
            name = curName + u"; " + unicode(name)
            self.toLine.setText(name)
 
-        self.recipients.append(data)
+    def takeContact(self,  contact,  phone):
+        to = unicode(self.toLine.text())
+        
+        name = contact.name()
+        if phone:
+            name += " (" + phone + ")"
+        
+        toList = to.split(";")
+        toList = [entry.strip() for entry in toList]
+        toList.remove(name)
+        to = "; ".join(toList)
+        
+        self.toLine.setText(to)
 
     def textChanged(self):
         len = int(self.messageText.toPlainText().length())
@@ -511,7 +641,23 @@ class MainWindow(QMainWindow,  ui.ui_main.Ui_MainWindow):
 
         self.checkSendButton()
 
-    def recipentChanged(self,  recipent):
+    def recipientChanged(self,  recipients):
+        # This is only called if the to line is changed by the user (and NOT programmatically)
+        toList = recipients.split(";")
+        toList = [unicode(entry).strip() for entry in toList]
+        
+        hideCell = self.settings.setting("contacts/hideCellnumber")
+        
+        for itemPos in xrange(self.contactsTree.topLevelItemCount()):
+            item = self.contactsTree.topLevelItem(itemPos)
+            contact = item.data(0,  Roles.ContactRole).toPyObject()
+            if (hideCell and contact.name() in toList) \
+                or (not hideCell and contact.name() + " (" + item.text(1) + ")" in toList):
+            
+                item.setIcon(0,  self.contactIsRecipientIcon)
+            else:
+                item.setIcon(0,  self.emptyIcon)
+        
         self.checkSendButton()
 
     def sendMessage(self):
@@ -539,7 +685,7 @@ class MainWindow(QMainWindow,  ui.ui_main.Ui_MainWindow):
 
             #  Only a name, sth. like foo
             else:
-                for recipient in self.recipients:
+                for recipient in self.database.contacts(True):
                     if unicode(recipient.name()) == name:
                         contact = copy.deepcopy(recipient)
                         if len(recipient.value("mobile_number")) > 1:
@@ -551,19 +697,6 @@ class MainWindow(QMainWindow,  ui.ui_main.Ui_MainWindow):
                                 continue
                         else:
                             contact.addInternalValue("phone",  recipient.value("mobile_number")[0])
-                if not contact:
-                    for recipient in self.database.contacts(True):
-                        if unicode(recipient.name()) == name:
-                            contact = copy.deepcopy(recipient)
-                            if len(recipient.value("mobile_number")) > 1:
-                                self.log.info(QString("Contact %1 has more then one mobile number.").arg(name))
-                                number = self.askMobileNumber(contact)
-                                if number != None:
-                                    contact.addInternalValue("phone",  number)
-                                else:
-                                    continue
-                            else:
-                                contact.addInternalValue("phone",  recipient.value("mobile_number")[0])
 
                 if not contact:
                     # foo must be in the contact list
@@ -588,11 +721,14 @@ class MainWindow(QMainWindow,  ui.ui_main.Ui_MainWindow):
             message.setMessage(msg)
             self.connection.sendMessage(message)
 
-            self.recipients = list()
             self.toLine.clear()
             self.messageText.clear()
             self.messageText.setFocus()
 
+        for itemPos in xrange(self.contactsTree.topLevelItemCount()):
+            item = self.contactsTree.topLevelItem(itemPos)
+            item.setIcon(0,  self.emptyIcon)
+
     def askMobileNumber(self,  contact):
         dlg = QDialog(self)
         dialog = ui.ui_mobileNumberSelect.Ui_MobileNumberSelectDialog()
@@ -751,6 +887,8 @@ class MainWindow(QMainWindow,  ui.ui_main.Ui_MainWindow):
         self.queueMessages = anz
 
     def closeConnection(self):
+        self.connectionClosedByUser = True
+
         self.connection.closeConnection()
         self.fileWidget.closeConnection()
 
@@ -767,17 +905,51 @@ class MainWindow(QMainWindow,  ui.ui_main.Ui_MainWindow):
         self.refreshButton.setEnabled(False)
         self.connectBar.setValue(0)
 
-        self.addDevices()
+        self.updateDevices()
         self.checkEditContactButton()
         self.checkSendButton()
+        if not self.connectionClosedByUser:
+            self.log.debug(QString("Automatic connection establishment: connection closed by error, restarting timer..."))
+            self.loadAutomaticConnection()
+        else:
+            self.log.debug(QString("Automatic connection establishment: connection closed by user, timer is not started"))
+
+    def automaticConnectionTimerFired(self):
+        self.log.debug(QString("Timer for automatic connection establishment fired.. scanning for devices"))
+        self.deviceScanner.start()
+
+    def automaticConnectionScanStarted(self):
+        self.log.debug(QString("Automatic connection establishment: Device scan started"))
+        movie = QMovie(":/loading-2",  "",  self)
+        movie.setScaledSize(QSize(16, 16))
+        self.scanningMovie.setMovie(movie)
+        self.scanningMovie.setToolTip(self.tr("There is an active device scan for the automatic connection establishment"))
+        self.scanningMovie.movie().start()
+
+    def automaticConnectionFoundDevice(self,  address,  name,  deviceClass):
+        for device in self.database.devices():
+            if device.bluetoothAddress() == address:
+                self.log.info(QString("Automatic connection establishment: Matching device found, connecting..."))
+                self.deviceScanner.stop()
+                self.connectionAttemptByUser = False
+                self.__connectToDevice(device)
+
+    def automaticConnectionScanFinished(self):
+        self.log.debug(QString("Automatic connection establishment: Device scan finished"))
+        if self.scanningMovie.movie():
+            self.scanningMovie.movie().stop()
+        self.scanningMovie.setMovie(QMovie())
+        self.scanningMovie.setToolTip("")
 
     @pyqtSignature("")
     def checkEditContactButton(self):
         if self.connection.connected() and self.contactsList.selectedItems():
             self.contactAddButton.setEnabled(True)
+            self.contactsImportButton.setEnabled(True)
             self.contactEditButton.setEnabled(True)
         else:
             self.contactAddButton.setEnabled(False)
+            self.contactsImportButton.setEnabled(False)
             self.contactEditButton.setEnabled(False)
 
     def checkSendButton(self):
@@ -798,7 +970,8 @@ class MainWindow(QMainWindow,  ui.ui_main.Ui_MainWindow):
                 except:
                     pass
 
-            myChat = window.chat.Chat(None, self.main,  contact)
+            #myChat = window.chat.Chat(None, self.main,  contact)
+            self.main.chatManager.openChat(contact)
 
     def favoriteClicked(self,  action):
         type = action.property("type").toString()
@@ -820,11 +993,11 @@ class MainWindow(QMainWindow,  ui.ui_main.Ui_MainWindow):
 
     def reinstallService(self):
         self.closeConnection()
-        
+
         dlg = QDialog(self)
         dialog = ui.ui_connection_update_version.Ui_ConnectionUpdateVersionDialog()
         dialog.setupUi(dlg)
-        
+
         if lib.obex_wrapper.FOUND_OBEX:
             dialog.obexStack.setCurrentWidget(dialog.obexFoundWidget)
             self.log.info(QString("OBEX support was found, trying to send installation file to device"))
@@ -836,52 +1009,84 @@ class MainWindow(QMainWindow,  ui.ui_main.Ui_MainWindow):
             else:
                 dialog.operatingSystemStack.setCurrentWidget(dialog.windowsWidget)
 
-        device = self.devicesBox.itemData(self.devicesBox.currentIndex()).toPyObject()
-        self.connect(dialog.sendApplicationButton,  SIGNAL("clicked()"),  
-                                lambda : self.helper.sendFile(self,  device, self.main.applicationSis,  self.main.applicationSisRegexp))
-        self.connect(dialog.sendPythonButton,  SIGNAL("clicked()"),  
-                                lambda : self.helper.sendFile(self, device, self.main.pythonSis,  self.main.pythonSisRegexp))
+        self.connect(dialog.sendApplicationButton,  SIGNAL("clicked()"),  lambda : self.sendApplicationFile(dialog.py20Box.isChecked()))
+        self.connect(dialog.sendPythonButton,  SIGNAL("clicked()"),  lambda : self.sendPythonFile(dialog.py20Box.isChecked()))
         self.connect(dialog.openFolderButton,  SIGNAL("clicked()"),  self.helper.openFolder)
 
         dlg.exec_()
 
+    def sendApplicationFile(self,  usePy20):
+        if usePy20:
+            self.helper.sendFile(self,  self.devicesBox.currentDevice(), self.main.applicationSis_Py20)
+        else:
+            self.helper.sendFile(self,  self.devicesBox.currentDevice(), self.main.applicationSis_Py14)
+
+    def sendPythonFile(self,  usePy20):
+        if usePy20:
+            self.helper.sendFile(self, self.devicesBox.currentDevice(), self.main.pythonSis_Py20)
+        else:
+            self.helper.sendFile(self,  self.devicesBox.currentDevice(), self.main.pythonSis_Py14)
+
     def connectionFailed(self,  errno,  errmsg):
+        self.connectionClosedByUser = False
+
+        if not self.connectionAttemptByUser:
+           self.statusBar().showMessage(self.tr("Connection to device failed: %1 - %2").arg(errno).arg(errmsg), 6000)
+           return
+
         dlg = QDialog(self)
         dialog = ui.ui_connection_failed.Ui_ConnectionFailedDialog()
         dialog.setupUi(dlg)
-        
+
         dialog.errnoLabel.setText("<b>" + str(errno) + "</b>")
         dialog.errmsgLabel.setText("<b>" + errmsg + "</b>")
-        
+
         self.main.setupButtonBox(dialog.buttonBox)
         self.connect(dialog.reinstallButton,  SIGNAL("clicked()"),  self.reinstallService)
-        
+
         self.connect(dialog.buttonBox.button(QDialogButtonBox.Retry),  SIGNAL("clicked()"),  self.connectToDevice)
         self.connect(dialog.buttonBox.button(QDialogButtonBox.Cancel),  SIGNAL("clicked()"),  self.closeConnection)
-        
+
         dlg.exec_()
-    
+
     def connectionVersionMismatch(self,  deviceVersion,  pcVersion):
         dlg = QDialog(self)
         dialog = ui.ui_connection_version_mismatch.Ui_ConnectionVersionMismatchDialog()
         dialog.setupUi(dlg)
-        
+
         dialog.mobileVersionLabel.setText("<b>" + str(deviceVersion) + "</b>")
         dialog.pcVersionLabel.setText("<b>" + str(pcVersion) + "</b>")
-        
+
         self.main.setupButtonBox(dialog.buttonBox)
         self.connect(dialog.updateButton,  SIGNAL("clicked()"),  self.reinstallService)
-        
+
         self.connect(dialog.buttonBox.button(QDialogButtonBox.Cancel),  SIGNAL("clicked()"),  self.closeConnection)
-        
+
         dlg.exec_()
-    
+
+    def updateCheckError(self,  errorMessage):
+        self.statusBar().showMessage(self.tr("Update check failed: %1").arg(errorMessage),  5000)
+
+    def updateCheckNewVersion(self,  version,  message):
+        text = self.tr("The update to <b>%1</b> of Series60-Remote is available at <b>%2</b>. Would you like to get it?").arg(version, self.settings.setting("updateCheck/website").toString())
+        if message:
+            text += '<br /><br />' + self.tr("Update notes: %1").arg(message)
+
+        button = QMessageBox.information(self,  self.tr("New version detected"), text,  QMessageBox.Yes | QMessageBox.No | QMessageBox.Ignore,  QMessageBox.Yes)
+        if button == QMessageBox.Yes:
+            QDesktopServices.openUrl(self.settings.setting("updateCheck/website"))
+        elif button == QMessageBox.Ignore:
+            self.settings.setSetting("updateCheck/interval",  0)
+
     def openDonateWebsite(self):
         QDesktopServices.openUrl(QUrl("http://sourceforge.net/donate/index.php?group_id=204429"))
 
     def addContact(self):
         dlg = window.contacts_edit.ContactsEdit(self,  self.main)
 
+    def importContacts(self,  format):
+        dlg = window.contacts_import.ContactsImport(self,  self.main,  format)
+
     def showFavoriteDialog(self):
         dlg = window.favorites.Favorites(self,  self.main)
 
@@ -941,7 +1146,7 @@ class MainWindow(QMainWindow,  ui.ui_main.Ui_MainWindow):
         else:
             for popup in self.main.popups:
                 popup.close()
-            
+
             if hasattr(self.main,  "trayicon"):
                 self.main.trayicon.hide()
             if self.connection.connected():
